

package applications;

public class Ackermann
{
   /** compute Ackermann's function
     * @throws IllegalArgumentException when i or j < 1 */
   public static int ackermann(int i, int j)
   {
      if (i < 1 || j < 1)
         throw new IllegalArgumentException("i and j must be >= 1, they are "
                                            + "i = " + i + "  j = " + j);

      return a(i, j);
   }

   /** compute Ackermann's function
     * assumes i,j >= 1 */
   private static int a(int i, int j)
   {
      if (i == 1)
      {// compute 2^j
         int ans = 2;
         for (int k = 1; k < j; k++)
            ans *= 2;
         return ans;
      }

      if (j == 1)
         return a(i - 1, 2);

      // i and j >= 2
      return a(i - 1, a(i, j - 1));
   }
   
   public static void main(String [] args)
   {
      System.out.println("A(1,2) = " + ackermann(1,2));
      System.out.println("A(2,1) = " + ackermann(2,1));
      System.out.println("A(2,2) = " + ackermann(2,2));
   }
}
