
/** closest pair of points on a horizontal line */

package applications;

import utilities.*;

public class ClosestHorizontalPoints
{
   // top-level nested classes
   /** point in 2D */
   public static class Point
   {
      // data members
      double x, y;    // point coordinates

      // constructor
      public Point(double theX, double theY)
      {
         x = theX;
         y = theY;
      }
   }

   /** point with id, implements Comparable using x-coordinates */
   public static class Point1 extends Point
                       implements Comparable
   {
      // data member
      int id;         // point identifier
     
      // constructor
      public Point1(double theX, double theY, int theID)
      {
         super(theX, theY);
         id = theID;
      }

      // method of comparable
   
      /** @return -1 if this.x < x.x,
        *          0 if this.x == x.x,
        *          1 if this.x > x.x */
      public int compareTo(Object x)
      {
         double xx = ((Point1) x).x;
         if (this.x < xx)
            return -1;
         if (this.x == xx)
            return 0;
         return 1;
      }

      public boolean equals(Object x)
         {return this.x == ((Point1) x).x;}
   }

   /** pairs of points and their distance */
   public static class PointPair
   {
      // data members
      Point1 a;       // one of the points
      Point1 b;       // the other point
      double dist;   // distance between a and b

      // constructor
      public PointPair(Point1 theA, Point1 theB, double theDist)
      {
         a = theA;
         b = theB;
         dist = theDist;
      }
   }
   
   /** @return distance between points u and v */
   public static double dist(Point u, Point v)
   {
      double dx = u.x - v.x;
      double dy = u.y - v.y;
      return Math.sqrt(dx * dx + dy * dy);
   }
   
   /** @return closest pair from n >= 2 points x[0:x.length-1]
     * that are on a horizontal line
     * @return null if fewer than 2 points
     * @exception throws IllegalArgumentException if the points
     * are not on a horizontal line */ 
   public static PointPair closestPair(Point1 [] x)
   {
      int n = x.length;  // number of points

      if (n < 2)
         return null;
   
      // verify that all points are on a horizontal line
      for (int i = 1; i < n - 1; i++)
         if (x[i].y != x[i + 1].y)
            throw new IllegalArgumentException
                  ("points are not on a horizontal line");
   
      // sort on x-coordinate
      MergeSort.mergeSort(x);
   
      // find closest adjacent points
      double minDistance = x[1].x - x[0].x;
      int minPoint = 0;
      for (int i = 1; i < n - 1; i++)
         // see if points i and i+1 are closer
         if (x[i+1].x - x[i].x < minDistance)
         {// closer pair
            minPoint = i;
            minDistance = x[i+1].x - x[i].x;
         }

      return new PointPair(x[minPoint], x[minPoint + 1], minDistance);
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      System.out.println("Enter number of points");
      int n = keyboard.readInteger();
      Point1 [] x = new Point1 [n];

      for (int i = 0; i < n; i++)
      {
         System.out.println("Enter point " + (i + 1));
         double xcoord = keyboard.readDouble();
         double ycoord = keyboard.readDouble();
         x[i] = new Point1(xcoord, ycoord, i + 1);
      }

      PointPair best = closestPair(x);
      System.out.println("Closest points are " + best.a.id +
                         " and " + best.b.id);
      System.out.println("Their distance is " + best.dist);
   }
}
