
/** first fit bin packing */

package applications;

import dataStructures.*;
import utilities.*;

public class FirstFit2
{
   // top-level nested class
   public static class Bin implements Playable
   {
      // data members
      int unusedCapacity;
   
      // constructor
      public Bin(int theUnusedCapacity)
         {unusedCapacity = theUnusedCapacity;}
   
      // method of Playable
      public boolean winnerOf(Playable theBin)
      {// define for a max winner tree
         if (unusedCapacity >= ((Bin) theBin).unusedCapacity)
            return true;
         else
            return false;
      }
   }
   
   /** output first fit packing into bins of size binCapacity
     * @param objectSize[1:objectSize.length-1] are the object sizes */
   public static void firstFitPack(int [] objectSize, int binCapacity)
   {
      int n = objectSize.length - 1;         // number of objects
      Bin [] bin = new Bin [n + 1];          // bins
      CompleteWinnerTree2 winTree = new CompleteWinnerTree2();
      
      // initialize n bins and winner tree
      for (int i = 1; i <= n; i++)
         bin[i] = new Bin(binCapacity);  // initial unused capacity
      winTree.initialize(bin);
      
      // put objects in bins
      for (int i = 1; i <= n; i++)
      {// put objectSize[i] in a bin
         // find first bin with enough capacity
         int pp = winTree.root();
         int p = winTree.leftChild(pp);       // pp is parent of p
         int gp = 0;                          // grandparent of p
         while (p != 0)
         {
            int winp = winTree.getWinner(p);
            if (bin[winp].unusedCapacity < objectSize[i])
               // first bin is in right subtree
               p = winTree.rightChild(pp);
            // move to left child of p
            gp = pp;
            pp = p;
            p = winTree.leftChild(pp);
         }
   
         int binToUse;   // will be set to bin to use
         if (pp != 0)
         {// at a tree node
            binToUse = winTree.getWinner(pp);
            // if binToUse is right child, need to check
            // bin binToUse-1.  No harm done by checking
            // bin binToUse-1 even if binToUse is left child.
            if (binToUse > 1 &&
                bin[binToUse-1].unusedCapacity >= objectSize[i])
               binToUse--;
         }
         else  // arises when n is odd
            binToUse = winTree.getWinner(p / 2);

         System.out.println("Pack object " + i + " in bin " + binToUse);
         bin[binToUse].unusedCapacity -= objectSize[i];
         winTree.rePlay(binToUse);
      }
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      System.out.println("Enter number of objects and bin capacity");
      int n = keyboard.readInteger();   // number of objects
      int binCapacity = keyboard.readInteger();
      if (n < 2)
      {
         System.out.println("Too few objects");
         System.exit(1);
      }

      // input the object sizes objectSize[1:n]
      int [] objectSize = new int [n + 1];
      for (int i = 1; i <= n; i++)
      {
         System.out.println("Enter space requirement of object " + i);
         objectSize[i] = keyboard.readInteger();
         if (objectSize[i] > binCapacity)
         {
           System.out.println("Object too large to fit in a bin");
           System.exit(1);
         }
      }

System.out.println("Donewith input");
      // output the packing
      firstFitPack(objectSize, binCapacity);
   }
}
