/** greedy machine scheduling using algorithm of Example 13.5 */

package applications;

import utilities.*;
import dataStructures.*;

public class GreedySchedule
{
   // top-level nested classes  
   private static class Task implements Comparable
   {
      int start;      // task start time
      int finish;     // task finish time
      int id;         // job identifier

      // constructor
      public Task(int theId, int theStart, int theFinish)
      {
         id = theId;
         start = theStart;
         finish = theFinish;
      }

      // define method of Comparable
      public int compareTo(Object x)
      {
         int xStart = ((Task) x).start;
         if (start < xStart)
            return -1;
         if (start == xStart)
            return 0;
         return 1;
      }

      public boolean equals(Object x)
         {return start == ((Task) x).start;}
   }
   
   private static class Machine implements Comparable
   {
      int id;          // machine identifier
      int avail;       // when it becomes free

      // constructor
      public Machine(int theId, int theAvail)
      {
         id = theId;
         avail = theAvail;
      }

      // define method of Comparable
      public int compareTo(Object x)
      {
         int xAvail = ((Machine) x).avail;
         if (avail < xAvail)
            return -1;
         if (avail == xAvail)
            return 0;
          return 1;
      }

      public boolean equals(Object x)
         {return avail == ((Machine) x).avail;}
   }

   /** greedy scheduler */
   public static void main(String [] args)
   {// Output an assignment of tasks to machines using
    // the fewest number of machine.

      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();
   
      // input the number of tasks
      System.out.println("Enter the number of tasks");
      int n = keyboard.readInteger();
      if (n < 1)
      {
         System.out.println("Must be more than 0");
         System.exit(1);
      }
   
      // input and store the n tasks in a task array
      Task [] t = new Task [n + 1];
      for (int i = 1; i <= n; i++)
      {
         System.out.println("Enter the start and finish time of task "
              + i);
         t[i] = new Task(i,
                         keyboard.readInteger(),
                         keyboard.readInteger());
         if (t[i].start < 0 || t[i].finish <= 0
             || t[i].start >= t[i].finish)
         {
             System.out.println("Bad start and/or finish time");
             System.exit(1);
         }
      }
   
      // sort tasks into ascending order of start times
      HeapSort.heapSort(t);
   
      MinHeapWithChangeMin minHeap = new MinHeapWithChangeMin(n);
   
      // we need at least one machine, put machine 1 in the min heap
      int machinesUsed = 1;
      Machine m = new Machine(1, 0);
      minHeap.put(m);

      // assign tasks to machines
      for (int i = 1; i <= n; i++)
      {
         // find first available machine
         m = (Machine) minHeap.getMin();
         if (m.avail > t[i].start)
         {// cannot use machine m, start a new one
            m = new Machine(++machinesUsed, t[i].finish);
            minHeap.put(m);
         }
         else
         {// use old machine
            m.avail = t[i].finish;
            minHeap.changeMin(m);
         }

         System.out.println("Assign task " + t[i].id + " to "
              + "machine " + m.id + " from "
              + t[i].start + " to " + t[i].finish);
      }
   }
}
