/** greedy algorithm to fold a stack of cartons */

package applications;

import utilities.*;


public class GreedyStackFolding
{
   /** @param h h[0:h.length - 1] is the array of carton heights
     * all heights are assumed to be > 0
     * @param height is maximum allowable height of a folded substack
     * @return an array substack such that substack[i] is
     * the substack into which carton i is folded
     * @return null iff h[i] > height for some i */
   public static int [] fold(double [] h, double height)
   {
      int [] substack = new int [h.length];
      int currentSubstack = 0;      // start with substack 0
      double currentHeight = 0;     // height of current substack

      // fold cartons into substacks in the order 0, 1, ...
      for (int i = 0; i < h.length; i++)
      {// pack carton i
         if (h[i] > height)
            // folding not possible
            return null;

         if (currentHeight + h[i] > height)
         {// start a new substack
            substack[i] = ++currentSubstack;
            currentHeight = h[i];
         }
         else
         {// put carton i into current substack
            substack[i] = currentSubstack;
            currentHeight += h[i];
         }
      }

      return substack;
   }
      
   /** test program */
   public static void main(String [] args)
   {
      // define a MyInputStream object to input from System.in
      MyInputStream keyboard = new MyInputStream();

      while (true)
      {
         System.out.println("Enter number of cartons, enter a "
                            + "number < 1 to terminate");
         int n = keyboard.readInteger();
         if (n < 1)
            break;

         // create the height array
         double [] h = new double [n];

         // input the stack heights
         System.out.println("Enter the stack heights");
         for (int i = 0; i < n; i++)
         {
            h[i] = keyboard.readDouble();
            if (h[i] <= 0)
            {// bad height
               System.out.println("All heights must be > 0");
               System.exit(1);
             }
         }

         // input the maximum allwable height for a substack
         System.out.println("Enter maximum allowable substack height");
         double maxHeight = keyboard.readDouble();
 
         // do the folding
         int [] substack = fold(h, maxHeight);

         // output fold points
         if (substack == null)
            System.out.println("Folding is not possible because "
                      + "one or more cartons have height more than "
                      + "the maximum allowable substack height\n");
         else
         {// feasible folding
            System.out.println("The assignment of cartons to substacks is: ");
            for (int i = 0; i < n; i++)
               System.out.print(substack[i] + " ");
            System.out.println("\n");
         }
      }
   }
}
