   
/** iterative version of dynamic programming solution for
  * the matrix multiplication chains problem, use upper
  * triangular matrices for c and kay */

package applications;

import dataStructures.*;
import utilities.*;

public class IterativeDPMatrixChainUsingUpperTriangularMatrices
{
   /** compute costs and kay for all Mij's iteratively
     * @param r[] is array of dimensions
     * @param c is cost matrix
     * @param kay is recurrence selectors */
   public static void matrixChain(int [] r, UpperTriangularMatrix c,
                                            UpperTriangularMatrix kay)
   {
      int q = r.length - 2;   // number of matrices
   
      // initialize c[i][i], c[i][i+1], and kay[i][i+1]
      for (int i = 1; i < q; i++)
      {
         c.set(i, i, new Integer(0));
         c.set(i, i + 1, new Integer(r[i] * r[i + 1] * r[i + 2]));
         kay.set(i, i + 1, new Integer(i));
      }
      c.set(q, q, new Integer(0));
      
      // compute remaining c's and kay's
      for (int s = 2; s < q; s++)
         for (int i = 1; i <= q - s; i++)
         {// min term for k = i
            int cValue = ((Integer) c.get(i + 1, i + s)).intValue()
                         + r[i] * r[i + 1] * r[i + s + 1];
            int kValue = i;
   
            // remaining min terms
            for (int k = i + 1; k < i + s; k++)
            {
               int t = ((Integer) c.get(i, k)).intValue()
                       + ((Integer) c.get(k + 1, i + s)).intValue()
                       + r[i] * r[k + 1] * r[i + s + 1];
               if (t < cValue)
               {// smaller min term, update c and kay
                  cValue = t;
                  kValue = k;
               }
            }      
            c.set(i, i + s, new Integer(cValue));
            kay.set(i, i + s, new Integer(kValue));
         }
   }
   
   /** output best way to compute Mij */
   public static void traceback(UpperTriangularMatrix kay, int i, int j)
   {
      if (i == j)  // only one matrix
         return;
      traceback(kay, i, ((Integer) kay.get(i, j)).intValue());
      traceback(kay, ((Integer) kay.get(i, j)).intValue() + 1, j);
      System.out.println("Multiply M " + i + ", " + kay.get(i, j) +
               " and M " + (((Integer) kay.get(i, j)).intValue() + 1)
               + ", " + j);
   }
   
   /** driver program */
   public static void main(String [] args)
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      // initialize
      System.out.println("Enter number of matrices");
      int q = keyboard.readInteger();
      int [] r = new int [q + 2];
      UpperTriangularMatrix kay = new UpperTriangularMatrix(q, new Integer(0));
      UpperTriangularMatrix c = new UpperTriangularMatrix(q, new Integer(0));

      // input matrix data
      for (int i = 1; i <= q; i++)
      {
         System.out.println("Enter number of rows in matrix " + i);
         r[i] = keyboard.readInteger();
      }
      System.out.println("Enter number of columns in matrix " + q);
      r[q + 1] = keyboard.readInteger();

      // compute cost of best way to multiply
      matrixChain(r, c, kay);
      System.out.println("Minimum cost is " + c.get(1, q));

      System.out.println( "\nCost matrix is");
      for (int i = 1; i <= q; i++)
      {
         for (int j = 1; j < i; j++)
            System.out.print(0 + " ");
         for (int j = i; j <= q; j++)
            System.out.print(c.get(i, j) + " ");
         System.out.println();
      }
      
      System.out.println("\nkay matrix is");
      for (int i = 1; i <= q; i++)
      {
         for (int j = 1; j <= i; j++)
            System.out.print(0 + " ");
         for (int j = i + 1; j <= q; j++)
            System.out.print(kay.get(i, j) + " ");
         System.out.println();
      }
      
      // output optimal multiplication sequence
      traceback(kay, 1, q);
   }
}
