/** compute last time a value was as big as it is now */

package applications;

import dataStructures.*;
import utilities.*;

public class LastAsBig
{
   /** @return a one-dimensional array lastAsBig such that
     * lastAsBig[i] gives the max j, j < i, such that
     * data[j] >= data[i]. lastAsBig[i] = -1 if there is
     * no such j. */
   public static int [] lastAsBig(Comparable [] data)
   {
      ArrayStack s = new ArrayStack();
      int length = data.length;
      int [] lastAsBig = new int [length];
   
      // compute lastAsBig[0:length-1]
      for (int i = 0; i < length; i++)
      {
         // unstack values < data[i]
         while (!s.empty() &&
                (data[((Integer) s.peek()).intValue()].
                 compareTo(data[i]) < 0))
            s.pop();

         // lastAsBig is given by top entry
         if (s.empty())
            // this big a value not seen before
            lastAsBig[i] = -1;
         else
         {// this big a value was seen before
            lastAsBig[i] = ((Integer) s.peek()).intValue();
            if (data[lastAsBig[i]].compareTo(data[i]) == 0)
               // don't need this on stack any more
               s.pop();
         }

         // i may be lastAsBig value for a future data, so stack it
         s.push(new Integer(i));
      }
      return lastAsBig;
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      // try 3 data arrays
      for (int i = 0; i < 3; i++)
      {
         System.out.println("Type number of data");
         int length = keyboard.readInteger();
         Integer [] data = new Integer [length];
   
         // input the data
         System.out.println("Type the data values");
         for (int j = 0; j < length; j++)
            data[j] = new Integer(keyboard.readInteger());
   
         // output the data
         System.out.println("The data values are");
         for (int j = 0; j < length; j++)
            System.out.print(data[j] + " ");
         System.out.println();

         // compute lastAsBig
         int [] lastAsBig = lastAsBig(data);

         // output lastAsBig
         System.out.println("The lastAsBig values are");
         for (int j = 0; j < length; j++)
            System.out.print(lastAsBig[j] + " ");
         System.out.println("\n");
      }
   }
}
