/** nonrecursive quick sort */

package applications;

import dataStructures.*;
import utilities.*;

public class QuickSort2
{
   /** data type for stack elements */
   static class Segment
   {
      int left;   // left end of segment
      int right;  // right end of segment

      // constructor
      public Segment(int theLeft, int theRight)
      {
         left = theLeft;
         right = theRight;
      }
   }


   /** sort the elements a[0 : a.length - 1] using the quick sort method */
   public static void quickSort(Comparable [] a)
   {
      if (a.length <= 1) return;
      // move largest element to right end
      MyMath.swap(a, a.length - 1, MyMath.max(a, a.length - 1));
   
      // define a stack for segments yet to be sorted
      ArrayStack stack = new ArrayStack();
      Segment s;
   
      // define left and right ends of current segment
      int leftEnd = 0;
      int rightEnd = a.length - 2;
   
      // sort segments until no segments remain to be sorted
      while (true)
      {
         // make sure current segment has > 1 element
         if (leftEnd >= rightEnd)
         {// current segment has <= 1 element
            // get another segment
            try {s = (Segment) stack.pop();}
            catch (Exception e)
            {// no segment remains
               return;
            }

            leftEnd = s.left;
            rightEnd = s.right;
         }
   
         int leftCursor = leftEnd,         // left-to-right cursor
             rightCursor = rightEnd + 1;   // right-to-left cursor
         Comparable pivot = a[leftEnd];
   
         // swap elements >= pivot on left side
         // with elements <= pivot on right side
         while (true)
         {
            do
            {// find >= element on left side
               leftCursor++;
            } while (a[leftCursor].compareTo(pivot) < 0);
   
            do
            {// find <= element on right side
               rightCursor--;
            } while (a[rightCursor].compareTo(pivot) > 0);
   
            if (leftCursor >= rightCursor) break;  // swap pair not found
            MyMath.swap(a, leftCursor, rightCursor);
         }

         // place pivot
         a[leftEnd] = a[rightCursor];
         a[rightCursor] = pivot;
      
         // put larger segment on stack and make smaller
         // one current segment
         if (rightCursor - leftEnd < rightEnd - rightCursor)
         {// left segment is smaller
            // stack right segment if it has > 1 element
            if (rightCursor + 1 < rightEnd)
               stack.push(new Segment(rightCursor + 1, rightEnd));
            rightEnd = rightCursor - 1;
         }
         else
         {// right segment is smaller
            // stack left segment if it has > 1 element
            if (leftEnd < rightCursor - 1)
               stack.push(new Segment(leftEnd, rightCursor - 1));
            leftEnd = rightCursor + 1;
         }
      }
   }
   
   /** test program */
   public static void main(String [] args)
   {
      Integer [] a = {new Integer(3),
                      new Integer(2),
                      new Integer(4),
                      new Integer(1),
                      new Integer(6),
                      new Integer(9),
                      new Integer(8),
                      new Integer(7),
                      new Integer(5),
                      new Integer(0)};

      // output elements to be sorted
      System.out.println("The elements are");
      for (int i = 0; i < a.length; i++)
         System.out.print(a[i] + " ");
      System.out.println();

      // sort the elements
      quickSort(a);

      // output in sorted order
      System.out.println("The sorted order is");
      for (int i = 0; i < a.length; i++)
         System.out.print(a[i] + " ");
      System.out.println();
   }
}

