
/** railroad car rearrangement using queues of finite size */

package applications;

import dataStructures.*;

public class RailroadWithQueuesAndTrackCapacities
{
   // data members
   private static ArrayQueue [] track;    // array of holding tracks
   private static int numberOfCars;
   private static int numberOfTracks;
   private static int smallestCar;        // smallest car in any holding track
   private static int itsTrack;           // holding track with car smallestCar
   private static int [] trackCapacity;   // array of track capacities
   private static int [] numberInTrack;   // number of cars in a holding track

   /** output the smallest car from the holding tracks */
   private static void outputFromHoldingTrack()
   {
      // remove smallestCar from itsTrack
      track[itsTrack].remove();
      numberInTrack[itsTrack]--;
      System.out.println("Move car " + smallestCar + " from holding track "
                         + itsTrack + " to output track");
   
      // find new smallestCar and itsTrack by checking top of all stacks
      smallestCar = numberOfCars + 2;
      for (int i = 1; i <= numberOfTracks; i++)
         if (!track[i].isEmpty() &&
             ((Integer) track[i].getFrontElement()).intValue() < smallestCar)
         {
            smallestCar = ((Integer) track[i].getFrontElement()).intValue();
            itsTrack = i;
         }
   }
   

  /** put car c into a holding track
    * @return false iff there is no feasible holding track for this car */
   private static boolean putInHoldingTrack(int c)
   {
      // find best holding track for car c
      // initialize
      int bestTrack = 0,  // best track so far
          bestLast =  0;  // last car in bestTrack
   
      // scan tracks
      for (int i = 1; i <= numberOfTracks; i++)
         if (!track[i].isEmpty() && numberInTrack[i] < trackCapacity[i])
         {// track i not empty and has space for more cars
             int lastCar = ((Integer) track[i].getRearElement()).intValue();
             if (c > lastCar && lastCar > bestLast)
             {
                // track i has bigger car at its rear
                bestLast = lastCar;
                bestTrack = i;
             }
         }
         else // track i may be a candidate
            if (bestTrack == 0 && numberInTrack[i] < trackCapacity[i])
               bestTrack = i;
         
      if (bestTrack == 0) return false; // no feasible track
   
      // add c to bestTrack
      track[bestTrack].put(new Integer(c));
      numberInTrack[bestTrack]++;
      System.out.println("Move car " + c + " from input track "
                         + "to holding track " + bestTrack);
   
      // update smallestCar and itsTrack if needed
      if (c < smallestCar)
      {   
          smallestCar = c;
          itsTrack = bestTrack;
      }
   
      return true;
   }
   
  /** rearrange railroad cars beginning with the initial order
    * inputOrder[1:theNumberOfCars]
    * the capacity of track i is theTrackCapacity[i], i > 0
    * @return true if successful, false if impossible. */
   public static boolean railroad(int [] inputOrder,
                         int theNumberOfCars, int theNumberOfTracks,
                         int [] theTrackCapacity)
   {
      numberOfCars = theNumberOfCars;
      trackCapacity = theTrackCapacity;
      numberOfTracks = theNumberOfTracks;
      numberInTrack = new int [numberOfTracks + 1];

      // find track with least capacity
      int minTrack = 1;
      for (int i = 2; i <= theNumberOfTracks; i++)
         if (trackCapacity[minTrack] > trackCapacity[i])
            minTrack = i;
      
      // keep minTrack open for direct output
      int minTrackCapacity = trackCapacity[minTrack];
      trackCapacity[minTrack] = 0;

      // create queues track[1:numberOfTracks] for use as holding tracks
      track = new ArrayQueue [numberOfTracks + 1];
      for (int i = 1; i <= numberOfTracks; i++)
         track[i] = new ArrayQueue();
   
      int nextCarToOutput = 1;
      smallestCar = numberOfCars + 1;  // no car in holding tracks
   
      // rearrange cars
      for (int i = 1; i <= numberOfCars; i++)
         if (inputOrder[i] == nextCarToOutput)
         {// send car inputOrder[i] straight out
             System.out.println("Move car " + inputOrder[i] + " from input "
                                + "track to output track");
             nextCarToOutput++;
    
             // output from holding tracks
             while (smallestCar == nextCarToOutput)
             {
                outputFromHoldingTrack();
       	        nextCarToOutput++;
            }
         }
         else
         // put car inputOrder[i] in a holding track
            if (!putInHoldingTrack(inputOrder[i]))
               return false;

      // reset minTrack capacity
      trackCapacity[minTrack] = minTrackCapacity;

      return true;
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // int [] p = {0, 5, 8, 1, 7, 4, 2, 9, 6, 3};
      int [] p = {0, 3, 6, 9, 2, 4, 1, 7, 8, 5};
      int [] capacity = {0, 3, 1, 2};
      System.out.println("Input permutation is 369241785");
      railroad(p, 9, 3, capacity);
   }
}
