/** find and ouput a path in a maze, moves to nearest 8 neighbors permitted */

package applications;

import java.util.*;
import utilities.*;

public class RatInAMaze2
{
   // top-level nested class
   private static class Position
   {
      // data members
      private int row;   // row number of the position
      private int col;   // column number of the position

      // constructor
      private Position(int theRow, int theCol)
      {
         row = theRow;
         col = theCol;
      }

      // convert to string suitable for output
      public String toString()
      {
         return new String(row + " " + col);
      }
   }

   // another top-level nested class
   private static class StackObject
   {
      // data members
      private Position pos; 
      private int option;   // option used to get to this position

      // constructor
      private StackObject(Position thePos, int theOption)
      {
         pos = thePos;
         option = theOption;
      }
   }
   
   // data members
   private static byte [][] maze;
   private static int size;   // number of rows and columns in the maze
   private static Stack path;  // path to current position
   
   // methods
   /* not yet implemented */
   private static void welcome() {};
   
   /** input the maze */
   private static void inputMaze()
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      System.out.println("Enter maze size");
      size = keyboard.readInteger();

      // create and input the maze array
      maze = new byte [size + 2][size + 2];
      System.out.println("Enter the maze in row-major order");
      for (int i = 1; i <= size; i++)
         for (int j = 1; j <= size; j++)
             maze[i][j] = keyboard.readByte();
   }
   
   /** find a path from (1,1) to the exit (size, size)
     * @return true if successful, false if impossible */
   private static boolean findPath()
   {
      path = new Stack();
   
      // initialize offsets
      Position [] offset = new Position [8];
      offset[0] = new Position(0, 1);   // east
      offset[1] = new Position(1, 1);   // south east
      offset[2] = new Position(1, 0);   // south
      offset[3] = new Position(1, -1);  // south west
      offset[4] = new Position(0, -1);  // west
      offset[5] = new Position(-1, -1); // north west
      offset[6] = new Position(-1, 0);  // north
      offset[7] = new Position(-1, 1);  // north east
      
      // initialize wall of obstacles around maze
      for (int i = 0; i <= size + 1; i++)
      {
         maze[0][i] = maze[size + 1][i] = 1; // bottom and top
         maze[i][0] = maze[i][size + 1] = 1; // left and right
      }
   
      Position here = new Position(1, 1);
      maze[1][1] = 1; // prevent return to entrance
      int option = 0; // next move
      int lastOption = 7;
      
      // search for a path
      while (here.row != size || here.col != size)
      {// not at exit
          // find a neighbor to move to
          // won't compile without explicit initialization
          int r = 0, c = 0;   // row and column of neighbor
          while (option <= lastOption)
          {
             r = here.row + offset[option].row;
             c = here.col + offset[option].col;
             if (maze[r][c] == 0) break;
             option++; // next option
          }
    
          // was a neighbor found?
          if (option <= lastOption)  // yes
          {// move to maze[r][c]
              path.push(new StackObject(here, option));
              here = new Position(r, c);
              // set to 1 to prevent revisit
              maze[r][c] = 1;
              option = 0;
          }
          else
          {// no neighbor to move to, back up
              if (path.empty()) return false;  // no place to backup to
              StackObject x = (StackObject) path.pop();
              here = x.pos;
              option = x.option + 1;
          }
      }
   
      return true;  // at exit
   }
   
   /** output path to exit */
   private static void outputPath()
   {
      System.out.println("The path is");
      while (!path.empty())
         System.out.println(((StackObject) path.pop()).pos);
   }
   
   /** entry point for rat in a maze program */
   public static void main(String [] args)
   {
      welcome();
      inputMaze();
      if (findPath()) outputPath();
      else System.out.println("No path");
   }
}
