/** loading containers on to two ships via backtracking
  * code finds best loading of first ship as well as its weight
  * this is done by first finding the weight of the best loading
  * and then determining the containers to load */


package applications;

public class RecursiveBTLoading5
{
   // class data members
   static int numberOfContainers;
   static int [] weight;
   static int capacity;
   static int weightOfCurrentLoading;
   static int maxWeightSoFar;
   static int [] bestLoadingSoFar;
   static int remainingWeight;

   
   
   /** @param theWeight array of container weights
     * @param theCapacity capacity of ship
     * set bestLoading[i] = 1 iff container i is loaded into ship 1
     * @return weight of best loading */
   public static int maxLoading(int [] theWeight, int theCapacity,
                                int [] bestLoading)
   {
      // set class data members
      numberOfContainers = theWeight.length - 1;
      weight = theWeight;
      capacity = theCapacity;
      bestLoadingSoFar = bestLoading;
      weightOfCurrentLoading = 0;
      maxWeightSoFar = 0;
      // initial remainingWeight is sum of all weights
      remainingWeight = 0;
      for (int i = 1; i <= numberOfContainers; i++) 
         remainingWeight += weight[i];

      // compute weight of best loading
      findBestWeight(1);

      // determine the best loading
      findBestLoading(1);

      return maxWeightSoFar;
   }

   /** method to find weight of best loading */
   private static void findBestWeight(int currentLevel)
   {// search from level currentLevel node
      if (currentLevel > numberOfContainers)
      {// at a leaf
         maxWeightSoFar = weightOfCurrentLoading;
         return;
      }

      // not at a leaf, check subtrees
      remainingWeight -= weight[currentLevel];
      if (weightOfCurrentLoading + weight[currentLevel] <= capacity)
      {// try x[currentLevel] = 1
         weightOfCurrentLoading += weight[currentLevel];
         findBestWeight(currentLevel + 1);
         weightOfCurrentLoading -= weight[currentLevel];
      }
      if (weightOfCurrentLoading + remainingWeight > maxWeightSoFar)
         // try left subtree
         findBestWeight(currentLevel+1);
      remainingWeight += weight[currentLevel];
   }
   
   /** actual method to find best loading 
     * search from a level currentLevel node for a loading
     * with weight maxWeightSoFar
     * @return true when such a loading is found */
   private static boolean findBestLoading(int currentLevel)
   {
      if (currentLevel > numberOfContainers)
         // at a leaf with load = maxWeightSoFar
         return true;

      // not at a leaf, check subtrees
      remainingWeight -= weight[currentLevel];
      if (weightOfCurrentLoading + weight[currentLevel] <= capacity)
      {// try left subtree
         weightOfCurrentLoading += weight[currentLevel];
         if (findBestLoading(currentLevel + 1))
         {// found loading with weight maxWeightSoFar in this subtree
            bestLoadingSoFar[currentLevel] = 1;
            return true;
         }
         weightOfCurrentLoading -= weight[currentLevel];
      }
      if (weightOfCurrentLoading + remainingWeight >= maxWeightSoFar)
         // try right subtree
         if (findBestLoading(currentLevel + 1))
         {// found loading with weight maxWeightSoFar in this subtree
            bestLoadingSoFar[currentLevel] = 0;
            return true;
         }
      remainingWeight += weight[currentLevel];
      return false;
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 5;
      int [] x = new int [n + 1];
      int [] w = {0, 7, 2, 6, 5, 4};
      int c = 10;
      System.out.println("Value of max loading for ship 1 is "
                         + maxLoading(w, c, x));
      System.out.println("The loading vector for ship 1 is ");
      for (int i = 1; i <= n; i++)
         System.out.print(x[i] + " ");
      System.out.println();
   }
}
