/** find and ouput a path in a maze, use recursion */

package applications;

import java.util.*;
import utilities.*;

public class RecursiveRatInAMaze
{
   // top-level nested class
   private static class Position
   {
      // data members
      private int row;   // row number of the position
      private int col;   // column number of the position

      // constructor
      private Position(int theRow, int theCol)
      {
         row = theRow;
         col = theCol;
      }

      // convert to string suitable for output
      public String toString()
      {
         return new String(row + " " + col);
      }
   }
   
   // data members
   private static byte [][] maze;
   private static int size;   // number of rows and columns in the maze
   private static Stack path;  // path to current position
   private static Position [] offset = new Position [4];
   private static int lastOption = 3;
   
   // methods
   /* not yet implemented */
   private static void welcome() {};
   
   /** input the maze */
   private static void inputMaze()
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      System.out.println("Enter maze size");
      size = keyboard.readInteger();

      // create and input the maze array
      maze = new byte [size + 2][size + 2];
      System.out.println("Enter the maze in row-major order");
      for (int i = 1; i <= size; i++)
         for (int j = 1; j <= size; j++)
             maze[i][j] = keyboard.readByte();
   }
   
   /** find a path from (1,1) to the exit (size, size)
     * @return true if successful, false if impossible */
   private static boolean findPath()
   {
      path = new Stack();
   
      // initialize offsets
      offset[0] = new Position(0, 1);   // right
      offset[1] = new Position(1, 0);   // down
      offset[2] = new Position(0, -1);  // left
      offset[3] = new Position(-1, 0);  // up
      
      // initialize wall of obstacles around maze
      for (int i = 0; i <= size + 1; i++)
      {
         maze[0][i] = maze[size + 1][i] = 1; // bottom and top
         maze[i][0] = maze[i][size + 1] = 1; // left and right
      }
   
      maze[1][1] = 1; // prevent return to (1, 1)

      return theFindPath(1, 1);
   }

   /** recursive method to find a path from (theRow, theCol) to (size, size) */
   private static boolean theFindPath(int theRow, int theCol)
   {
      if (theRow == size && theCol == size)
         // reached the exit
         return true;

      // not at exit
      for (int option = 0; option <= lastOption; option++)
      {
         int r = theRow + offset[option].row;
         int c = theCol + offset[option].col;
         if (maze[r][c] == 0)
         {// move to (r, c) and search for a path from (r, c)
            path.push(new Position(theRow, theCol));
            maze[r][c] = 1;
            if (theFindPath(r, c))
               return true;
            path.pop();
         }
      }

      return false;  // no path found
   }
   
   /** output path to exit */
   private static void outputPath()
   {
      System.out.println("The path is");
      while (!path.empty())
         System.out.println(path.pop());
   }
   
   /** entry point for rat in a maze program */
   public static void main(String [] args)
   {
      welcome();
      inputMaze();
      if (findPath()) outputPath();
      else System.out.println("No path");
   }
}
