
package applications;

import utilities.*;

public class Weight
{
   // class constants
   public static final boolean PLUS = true;
   public static final boolean MINUS = false;

   // instance data members
   private boolean sign;
   private long metricTons;
   private int kilograms;
   private int grams;

   // constructors
   /** @throws IllegalArgumentException when theMetricTons < 0
     * or theKilograms < 0 or theKilograms > 999
     * or theMilliMetricTons < 0 or theGrams > 999 */
   public Weight(boolean theSign, long theMetricTons,
                 int theKilograms, int theGrams)
   {
      sign = theSign;

      if (theMetricTons < 0)
         throw new IllegalArgumentException
               ("MetricTons value must be >= 0, it is " + theMetricTons);
      metricTons = theMetricTons;

      if (theKilograms < 0 || theKilograms > 999)
         throw new IllegalArgumentException
               ("Kilograms must be between 0 and 999, it is "
                + theKilograms);
      kilograms = theKilograms;

      if (theGrams < 0 || theGrams > 999)
         throw new IllegalArgumentException
               ("Grams must be between 0 and 999, it is "
                + theGrams);
      grams = theGrams;
   }

   /** initialize instance to 0 */
   public Weight()
      {this(PLUS, 0L, 0, 0);}

   /** initialize with weight = theValue metricTons */
   public Weight(double theValue)
      {setValue(theValue);}

   // accessor methods
   /** @return sign */
   public boolean getSign()
      {return sign;}

   /** @return metricTons */
   public long getMetricTons()
      {return metricTons;}

   /** @return kilograms */
   public int getKilograms()
      {return kilograms;}

   /** @return grams */
   public int getGrams()
      {return grams;}

   // mutator methods
   /** set sign = theSign */
   public void setSign(boolean theSign)
      {sign = theSign;}
 
   /** set metricTons = theMetricTons
     * @throws IllegalArgumentException when theMetricTons < 0 */
   public void setMetricTons(long theMetricTons)
   {
      if (theMetricTons < 0)
         throw new IllegalArgumentException
               ("MetricTons value must be >= 0, it is " + theMetricTons);
      metricTons = theMetricTons;
   }

   /** set kilograms = theKilograms
     * @throws IllegalArgumentException when theKilograms < 0 or > 999 */
   public void setKilograms(int theKilograms)
   {
      if (theKilograms < 0 || theKilograms > 999)
         throw new IllegalArgumentException
               ("Kilograms must be between 0 and 999, it is "
                + theKilograms);
      kilograms = theKilograms;
   }

   /** set grams = theGrams
     * @throws IllegalArgumentException when theGrams < 0 or > 999 */
   public void setGrams(int theGrams)
   {
      if (theGrams < 0 || theGrams > 999)
         throw new IllegalArgumentException
               ("Grams must be between 0 and 999, it is "
                + theGrams);
      grams = theGrams;
   }

   /** set length = theValue metricTons */
   public void setValue(double theValue)
   {
      if (theValue < 0)
      {
         sign = MINUS;
         theValue = -theValue;
      }
      else
         sign = PLUS;

      metricTons = (long) (theValue + 0.0000005); // extract integral part

      // get six decimal digits
      int six = (int) ((theValue + 0.0000005 - metricTons) * 1000000);

      kilograms = six / 1000;
      grams = six - kilograms * 1000;
   }

   public void setValue(Weight x)
   {
      sign = x.sign;
      metricTons = x.metricTons;
      kilograms = x.kilograms;
      grams = x.grams;
   }

   /** input a weight */
   public void input()
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      // input the weight as a double
      System.out.println("Enter the weight in metricTons as a real number");
      double theValue = keyboard.readDouble();

      // set the value
      setValue(theValue);
   }
   
   /** convert to a string */
   public String toString()
   {
      if (sign == PLUS)
         {return metricTons + "." + kilograms + grams;}
      else
         {return "-" + metricTons + "." + kilograms + grams;}
   }

   // arithmetic methods
   /** @return this + x */
   public Weight add(Weight x)
   {
      long a1, a2, a3;
      // convert this to a long
      a1 = metricTons * 1000000 + kilograms * 1000 + grams;
      if (sign == MINUS)
         a1 = -a1;
      
      // convert x to a long
      a2 = x.metricTons * 1000000 + x.kilograms * 1000 + x.grams;
      if (x.sign == MINUS)
         a2 = -a2;
      
      a3 = a1 + a2;
       
      // convert result to Weight object
      Weight answer = new Weight();
      if (a3 < 0)
      {
         answer.sign = MINUS;
         a3 = -a3;
      }
      else
         answer.sign = PLUS;
      answer.metricTons = a3 / 1000000;
      a3 -= answer.metricTons * 1000000;  // subtract the metricTons
      answer.kilograms = (int) (a3 / 1000);
      answer.grams = (int) (a3 - answer.kilograms * 1000);
      
      return answer;
   }
   
   /** @return this - x */
   public Weight subtract(Weight x)
   {
      long a1, a2, a3;
      // convert this to a long
      a1 = metricTons * 1000000 + kilograms * 1000 + grams;
      if (sign == MINUS)
         a1 = -a1;
      
      // convert x to a long
      a2 = x.metricTons * 1000000 + x.kilograms * 1000 + x.grams;
      if (x.sign == MINUS)
         a2 = -a2;
      
      a3 = a1 - a2;
       
      // convert result to Weight object
      Weight answer = new Weight();
      if (a3 < 0)
      {
         answer.sign = MINUS;
         a3 = -a3;
      }
      else
         answer.sign = PLUS;
      answer.metricTons = a3 / 1000000;
      a3 -= answer.metricTons * 1000000;  // subtract the metricTons
      answer.kilograms = (int) (a3 / 1000);
      answer.grams = (int) (a3 - answer.kilograms * 1000);
      
      return answer;
   }
   
   
   /** @return this * x */
   public Weight multiply(int x)
   {
      // convert this to a long
      long a1 = metricTons * 1000000 + kilograms * 1000 + grams;
      if (sign == MINUS)
         a1 = -a1;
      
      long a2 = a1 * x;
       
      // convert result to Weight object
      Weight answer = new Weight();
      if (a2 < 0)
      {
         answer.sign = MINUS;
         a2 = -a2;
      }
      else
         answer.sign = PLUS;
      answer.metricTons = a2 / 1000000;
      a2 -= answer.metricTons * 1000000;  // subtract the metricTons
      answer.kilograms = (int) (a2 / 1000);
      answer.grams = (int) (a2 - answer.kilograms * 1000);
      
      return answer;
   }
   
   /** @return value of this computed at the rate $x per kilogram */
   public Currency value(Currency x)
   {
      // convert this to a double
      double a1 = ((double) metricTons) * 1000 + kilograms
                         + ((double) grams) / 1000;
      if (sign == MINUS)
         a1 = -a1;
      
      // convert x to a double
      double a2 = x.getDollars() + ((double) x.getCents()) / 100;
      if (x.getSign() == MINUS)
         a2 = -a2;

      return new Currency(a1 * a2);
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // test constructors
      Weight g = new Weight(),
             h = new Weight(PLUS, 3L, 502, 421),
             i = new Weight(-2.507633),
             j = new Weight();

      // test input
      j.input();

      // test toString
      System.out.println("The initial values are " + g +
                         " " + h + " " + i + " " + j);
      System.out.println();

      // test mutators
      // first make g nonzero
      g.setMetricTons(2);
      g.setSign(MINUS);
      g.setKilograms(256);
      g.setGrams(123);
      i.setValue(-6.453210);
      System.out.println("New values are " + g + " " + i);
      System.out.println();

      // do some arithmetic
      j = h.add(g);
      System.out.println(h + " + " + g + " = " + j);

      j = i.add(g).subtract(h);
      System.out.println(i + " + " + g + " - " + h +
                         " = " + j);
      System.out.println();

      j = i.multiply(2);
      System.out.println(i + " * 2 = "  + j);
      System.out.println();

      Currency x = new Currency(3.0);
      Currency y = i.value(x);
      System.out.println("Value of " + i + " metricTons at $3 per "
                         + "kilogram is "  + y);
      System.out.println();
   }
}
