/** sort using a winner tree */

package applications;

import java.lang.reflect.*;
import dataStructures.*;
import utilities.*;
import wrappers.*;

public class WinnerTreeSort
{
   /** @param one is an object such that a[i].add(one) > a[i]
     * @return an array that contains the elements
     * a[1 : a.length - 1] using a winner tree */
   public static Operable [] winnerTreeSort(Operable [] a, Object one)
   {// a must also implement Playable
      if (a.length < 3)
         // less than 2 elements
         return a;
   
      // first find max element
      Operable maxElement = a[1];
      for (int i = 2; i < a.length; i++)
         if (maxElement.compareTo(a[i]) < 0)
            maxElement = a[i];
   
      // now create larger element for use as the infinity element
      // do not use increment because this will change an element in a[]
      maxElement = (Operable) maxElement.add(one); 
   
      // create an array to collect elements in sorted order
      Operable [] b = (Operable []) Array.newInstance
                           (a.getClass().getComponentType(), a.length);
   
      // create a min winner tree of the elements
      CompleteWinnerTree wTree = new CompleteWinnerTree();
      wTree.initialize((Playable[]) a);
   
      // extract one by one from the winner tree
      for (int i = 1; i < a.length; i++)
      {
         // determine next smallest element
         int w = wTree.getWinner();

         // move a[w] to proper place in sorted array
         b[i] = a[w];

         // replace a[w] with large element
         a[w] = maxElement;

         // adjust winner tree
         wTree.rePlay(w);
      }
      return b;
   }

   /** test program */
   public static void main(String [] args)
   {
      NewMyInteger [] a = {new NewMyInteger(3),
                           new NewMyInteger(2),
                           new NewMyInteger(4),
                           new NewMyInteger(1),
                           new NewMyInteger(6),
                           new NewMyInteger(9),
                           new NewMyInteger(8),
                           new NewMyInteger(7),
                           new NewMyInteger(5),
                           new NewMyInteger(0)};

      // output elements to be sorted
      System.out.println("The elements are");
      for (int i = 1; i < a.length; i++)
         System.out.print(a[i] + " ");
      System.out.println();

      // sort the elements
      a = (NewMyInteger []) winnerTreeSort(a, new NewMyInteger(1));

      // output in sorted order
      System.out.println("The sorted order is");
      for (int i = 1; i < a.length; i++)
         System.out.print(a[i] + " ");
      System.out.println();
   }
}

