
/** worst fit bin packing */

package applications;

import dataStructures.*;
import utilities.*;

public class WorstFit
{
   // top-level nested class
   public static class BinNode implements Comparable
   {
      // data members
      int id,     // bin identifier
          avail;  // available capacity

      // constructor
      public BinNode(int theId, int theAvail)
      {
         id = theId;
         avail = theAvail;
      }

      // define method of the Comparable interface
      public int compareTo(Object x)
      {
          if (avail < ((BinNode) x).avail)
             return -1;
          if (avail == ((BinNode) x).avail)
             return 0;
          return 1;
      }

      // override Object.equals
      public boolean equals(Object x)
         {return avail == ((BinNode) x).avail;}
   }
   
   /** output worst fit packing into bins of size c
     * @param s[1:s.length-1] are the object sizes */
   public static void worstFitPack(int [] s, int c)
   {
      int n = s.length - 1;             // number of objects
      int b = 0;                        // number of bins used
      MaxHeapWithChangeMax theHeap;     // max heap of bin capacities
      theHeap = new MaxHeapWithChangeMax(n);
      
      // pack objects one by one
      for (int i = 1; i <= n; i++)
      {// pack object i
         // find worst bin
         BinNode worstBin = (BinNode) theHeap.getMax();
         if (worstBin == null || worstBin.avail < s[i])
         {// no bin large enough, use a new bin
            System.out.println("Pack object " + i + " in bin " + (++b));
            // put bin in heap if bin is not full
            if (c > s[i])
               theHeap.put(new BinNode(b, c - s[i]));
         }
         else
         {// pack in worstBin
            System.out.println("Pack object " + i + " in bin " + worstBin.id);
   
            // update available capacity of worstBin
            worstBin.avail -= s[i];
            if (worstBin.avail > 0)
               theHeap.changeMax(worstBin);
            else
               theHeap.removeMax();
         }
      }
   }

   /** test program */
   public static void main(String [] args)
   {
      // define the input stream to be the standard input stream
      MyInputStream keyboard = new MyInputStream();

      System.out.println("Enter number of objects and bin capacity");
      int n = keyboard.readInteger();   // number of objects
      int c = keyboard.readInteger();   // bin capacity
      if (n < 2)
      {
         System.out.println("Too few objects");
         System.exit(1);
      }

      // input the object sizes s[1:n]
      int [] s = new int [n + 1];
      for (int i = 1; i <= n; i++)
      {
         System.out.println("Enter space requirement of object " + i);
         s[i] = keyboard.readInteger();
         if (s[i] > c)
         {
           System.out.println("Object too large to fit in a bin");
           System.exit(1);
         }
      }

      // output the packing
      worstFitPack(s, c);
   }
}
