
/** iterator for array-based implementation of LinearList
  * using Equation 5.3 */

package dataStructures;

import java.util.*;

public class ArrayCircularListWithIterator
             extends ArrayCircularList
{
   // constructors
   public ArrayCircularListWithIterator(int initialCapacity)
      {super(initialCapacity);}

   public ArrayCircularListWithIterator()
   {// use default capacity of 10
      this(10);
   }

   /** create and return an iterator */
   public Iterator iterator()
      {return new ArrayCircularListIterator();}

   private class ArrayCircularListIterator implements Iterator
   {
      // data members
      private int nextIndex;  // index of next element
   
      // constructor
      public ArrayCircularListIterator()
         {nextIndex =  first;}
   
      // methods
      /** @return true iff list has more elements */
      public boolean hasNext()
         {return nextIndex != -1;}
   
      /** @return next element in list
        * @throws NoSuchElementException
        * when there is no next element */
      public Object next()
      {
         if (nextIndex != -1)
         {
            Object temp = element[nextIndex];
            if (nextIndex == last)
               nextIndex = -1;
            else
              nextIndex = (nextIndex + 1) % element.length;
            return temp;
         }
         else throw new NoSuchElementException("No next element");
      }

      /** unsupported method */
      public void remove()
      {
         throw new UnsupportedOperationException
                   ("remove not supported");
      }   
   }

   /** test program */
   public static void main(String [] args)
   {
      // test default constructor
      ArrayCircularListWithIterator x =
                  new ArrayCircularListWithIterator();

      // insert a few elements
      x.add(0, new Integer(2));
      x.add(1, new Integer(6));
      x.add(0, new Integer(1));
      x.add(2, new Integer(4));
      System.out.println("The list is " + x);

      // test iterator
      System.out.print("The list is [");
      Iterator ix = x.iterator();
      while (ix.hasNext())
         System.out.print(ix.next() + " ");
      System.out.println("]");
   }
}
