/** split an array-based linear list into two
  * Equation 5.3 is used */

package dataStructures;

public class ArrayCircularListWithSplit
             extends ArrayCircularList
{
   // constructors
   public ArrayCircularListWithSplit(int initialCapacity)
      {super(initialCapacity);}

   public ArrayCircularListWithSplit()
   {// use default capacity of 10
      this(10);
   }

   /** split this into lists a and b */
   public void split(ArrayCircularListWithSplit a,
                     ArrayCircularListWithSplit b)
   {
      // compute size of each split list, note that bSize()
      // is 1 less than aSize() when this.size() is odd
      int bSize = size() / 2;
      int aSize = size() - bSize;

      // get large enough arrays
      a.element = new Object [aSize];
      b.element = new Object [bSize];
      
      int ct = first;         // cursor for this
      for (int i = 0; i < bSize; i++)
      {
         a.element[i] = element[ct];
         ct = (ct + 1) % element.length;
         b.element[i] = element[ct];
         ct = (ct + 1) % element.length;
      }
      
      // is an element left?
      if (aSize > bSize)
         a.element[aSize - 1] = element[ct];

      // set data members of a and b
      a.first = 0;
      a.last = aSize - 1;
      b.first = 0;
      b.last = bSize - 1;
   }

   /** test program */
   public static void main(String [] args)
   {
      ArrayCircularListWithSplit x =
                  new ArrayCircularListWithSplit();
      ArrayCircularListWithSplit y =
                  new ArrayCircularListWithSplit();
      ArrayCircularListWithSplit z =
                  new ArrayCircularListWithSplit();
      
      //  first try a list with even size
      int n = 10;
      // put elements in x
      for (int i = 0; i < n; i++)
         x.add(i, new Integer(i));
      
      System.out.println("The list is " + x);
      x.split(y, z);
      System.out.println("The split lists are " + y + "\nand " + z);
      System.out.println();
      
      //  now try a list with odd size
      // put one more element into x
      x.add(n, new Integer(n));
      
      System.out.println("The list is " + x);
      x.split(y, z);
      System.out.println("The split lists are " + y + "\nand " + z);
      System.out.println();
   }
}
