/** ArrayLinearList with capacity increment */

package dataStructures;

import java.util.*;
import utilities.*;

public class ArrayLinearListWithCapacityIncrement extends ArrayLinearList
{
   // data member
   int capacityIncrement;

   // constructors
   public ArrayLinearListWithCapacityIncrement(int initialCapacity)
      {super(initialCapacity);}

   /** create a list with initial capacity initialCapacity
     * @throws IllegalArgumentException when
     * initialCapacity < 1 or theCapacityIncrement < 1 */
   public ArrayLinearListWithCapacityIncrement(int initialCapacity,
                                               int theCapacityIncrement)
   {
      super(initialCapacity);
      if (theCapacityIncrement < 1)
         throw new IllegalArgumentException
                   ("capacityIncrement must be >= 1");
      capacityIncrement = theCapacityIncrement;
   }

   public ArrayLinearListWithCapacityIncrement()
   {// use default capacity of 10
      this(10);
   }

   /** Insert an element with specified index.
     * All elements with equal or higher index
     * have their index increased by 1.
     * @throws IndexOutOfBoundsException when
     * index is not between 0 and size */
   public void add(int index, Object theElement)
   {
      if (index < 0 || index > size)
         // invalid list position
         throw new IndexOutOfBoundsException
             ("index = " + index + "  size = " + size);

      // valid index, make sure we have space
      if (size == element.length)
         // no space, increase capacity
         if (capacityIncrement < 1)
            // double capacity
            element = ChangeArrayLength.changeLength1D
                         (element, 2 * element.length);
         else
            // increase capacity by capacityIcrement
            element = ChangeArrayLength.changeLength1D
                         (element, element.length + capacityIncrement);

      // shift elements right one position
      for (int i = size - 1; i >= index; i--)
         element[i + 1] = element[i];

      element[index] = theElement;

      size++;
   }

   /** test program */
   public static void main(String [] args)
   {
      // create a list whose capacity doubles
      ArrayLinearListWithCapacityIncrement x = 
         new ArrayLinearListWithCapacityIncrement(1);

      // capacity should double
      for (int i = 1; i < 10; i++)
      {
         x.add(0, new Integer(i));
         System.out.println("List capacity is " + x.element.length);
      }

      // create a list whose capacity increases by 1
      x = new ArrayLinearListWithCapacityIncrement(1, 1);

      // capacity should increase by 1
      for (int i = 1; i < 10; i++)
      {
         x.add(0, new Integer(i));
         System.out.println("List capacity is " + x.element.length);
      }
   }
}
