
/** circular shift (clockwise) an array-based linear list */

package dataStructures;

import utilities.*;

public class ArrayLinearListWithCircularShift
       extends ArrayLinearList
{
   /** reverse the segment element[start:finish] */
   private void reverse(int start, int finish)
   {
      int numberToSwap = (finish - start + 1) / 2;
      for (int i = 0; i < numberToSwap; i++)
         MyMath.swap(element, start++, finish--);
   }

   /** shift the elements clockwise circularly by shiftAmount
     * @throws IllegalArgumentException when
     * shiftAmount < 0 */
   private void circularShift(int shiftAmount)
   {
      // empty list is easily shifted by any amount
      if (size == 0)
         return;

      // validate shiftAmount
      if (shiftAmount < 0)
         throw new IllegalArgumentException
                   ("shift amount must be >= 0, it is " + shiftAmount);

      // get equivalent amount in the range 0 though size - 1
      shiftAmount %= size;
  
      // expedite zero shift, not necessary to do this
      if (shiftAmount == 0)
         return;

      // do the shift as three segment reversals
      reverse(0, size - 1);                    // reverse whole list
      reverse(0, size - shiftAmount - 1);      // reverse left segment
      reverse(size - shiftAmount, size - 1);   // reverse right segment
   }


   /** test program */
   public static void main(String [] args)
   {
      int numberOfSizes = 2;
      int [] size = {5, 8};   // list sizes to test
      
      for (int j = 0; j < numberOfSizes; j++)
      {
         int n = size[j];
         for (int k = 0; k <= n; k++)
         {// k is shift amount
            ArrayLinearListWithCircularShift x =
               new ArrayLinearListWithCircularShift();
            
            //  put elements in x
            for (int i = 0; i < n; i++)
               x.add(0, new Integer(n - i - 1));
            
            System.out.println("The list is " + x);
            x.circularShift(k);
            System.out.println("The list shifted by " + k + " is " + x);
            System.out.println();
         }   
         System.out.println();
      }
   }   
}
