
/** array-based implementation of LinearList that decreases the size
  * of the element array when occupancy drops below 25% */

package dataStructures;

import java.util.*;
import utilities.*;

public class ArrayLinearListWithDecreaseSize
             extends ArrayLinearList
{
   // additional data member
   protected int initialCapacity;

   // constructors
   /** create a list with initial capacity initialCapacity */
   public ArrayLinearListWithDecreaseSize(int initialCapacity)
   {
      super(initialCapacity);
      this.initialCapacity = initialCapacity;
   }

   /** create a list with initial capacity 10 */
   public ArrayLinearListWithDecreaseSize()
   {// use default capacity of 10
      this(10);
   }

   /** overrides ArrayLinearList.remove */
   public Object remove(int index)
   {
      Object elementToReturn = super.remove(index);
  
      // decrease size of element array if occupancy is below 25%
      if (size < element.length / 4 && element.length / 2 >= initialCapacity)
         // halve the size of the array
         element = ChangeArrayLength.changeLength1D
                      (element, size, element.length / 2);
      return elementToReturn;
   }

   /** test program */
   public static void main(String [] args)
   {
      LinearList x = new ArrayLinearListWithDecreaseSize(2);
      int n = 10;

      // insert n elements
      for (int i = 0; i < n; i++)
      {
         x.add(0, new Integer(i));
         System.out.println("The capacity is " +
                ((ArrayLinearList) x).element.length);
      }
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);


      // remove n-1 elements
      for (int i = 0; i < n - 1; i++)
      {
         System.out.println("Removed element " + x.remove(0));
         System.out.println("The capacity is " +
                ((ArrayLinearList) x).element.length);
      }
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
   }
}
