

/** extension of ArrayLinearList to include setSize */

package dataStructures;

import utilities.*;

public class ArrayLinearListWithSetSize extends ArrayLinearList
{
   // constructors
   public ArrayLinearListWithSetSize(int initialCapacity)
      {super(initialCapacity);}

   public ArrayLinearListWithSetSize()
   {// use default capacity of 10
      this(10);
   }

   /** make size = theSize
     * @throws IllegalArgumentException when
     * theSize < 0 */
   public void setSize(int theSize)
   {
      if (theSize < 1)
         throw new IllegalArgumentException
                   ("theSize must be >= 0, it is " + theSize);

      if (theSize <  size)
         // eliminate some elements
         while (theSize != size)
            element[--size] = null;
      else
      {
         if (theSize > element.length)
            // not enough capacity, increase capacity
            element = ChangeArrayLength.changeLength1D
                      (element, size, theSize);
         size = theSize;
      }
   }   

   /** test program */
   public static void main(String [] args)
   {
      ArrayLinearListWithSetSize x = new ArrayLinearListWithSetSize(1);

      x.add(0, new Integer(0));
      x.add(1, new Integer(1));
      System.out.println("List size is " + x.size());
      System.out.println("Array length is " + x.element.length);
      System.out.println("The list is " + x);
      System.out.println();

      x.setSize(6);
      System.out.println("List size is " + x.size());
      System.out.println("Array length is " + x.element.length);
      System.out.println("The list is " + x);
      System.out.println();

      x.setSize(3);
      System.out.println("List size is " + x.size());
      System.out.println("Array length is " + x.element.length);
      System.out.println("The list is " + x);
      System.out.println();
   }
}
