
/** backtracking salesperson problem with stronger condition */

package dataStructures;

import utilities.*;
import wrappers.*;

public class BTSalespersonWithStrongerCondition
             extends AdjacencyWDigraph
{
   // class data members
   static int [] partialTour;
   static int [] bestTourSoFar;
   static Object costOfBestTourSoFar;
   static Operable costOfPartialTour;
   static Operable [] minOut;
       // cost of least cost edge out of each vertex
   static Operable remainingSum;
       // sum of minOut[] values for remaining vertices

   // constructors
   public BTSalespersonWithStrongerCondition(int theVertices)
      {super(theVertices);}

   public BTSalespersonWithStrongerCondition()
      {this(0);}

   /** traveling salesperson by backtracking
     * @param theZero zero weight
     * @param bestTour bestTour[1:n] is set to best tour
     * @return cost of best tour */
   public Object btSalesperson3(int [] bestTour, Operable theZero)
   {
      // determine cost of least-cost out-edge from each vertex
      minOut = new Operable[n + 1];
      for (int i = 1; i <= n; i++)
         for (int j = 1; j <= n; j++)
            if (a[i][j] != null &&
                (minOut[i] == null || minOut[i].compareTo(a[i][j]) > 0))
               minOut[i] = (Operable) a[i][j];
   
      // make sure each vertex has an out edge
      for (int i = 1; i <= n; i++)
         if (minOut[i] == null)
            // there can be no tour
            return null;
   
      // determine sum of minOut[] of all vertices except vertex 1
      remainingSum = (Operable) theZero.zero();
      for (int i = 2; i <= n; i++)
         remainingSum.increment(minOut[i]);

      // set partialTour to identity permutation
      partialTour = new int [n + 1];
      for (int i = 1; i <= n; i++)
         partialTour[i] = i;

      costOfBestTourSoFar = null;
      bestTourSoFar = bestTour;
      costOfPartialTour = theZero;
   
      // search permutations of partialTour[2:n]
      rTSP(2);
   
      return costOfBestTourSoFar;
   }
   
   /** recursive backtracking code for traveling salesperson
     * search the permutation tree for best tour */
   private void rTSP(int currentLevel)
   {// search from a node at currentLevel
      if (currentLevel == n)
      {// at parent of a leaf
         // complete tour by adding last two edges
         if (a[partialTour[n - 1]][partialTour[n]] != null &&
             a[partialTour[n]][1] != null &&
             (costOfBestTourSoFar == null ||
             ((Operable) ((Operable) costOfPartialTour.
                           add(a[partialTour[n - 1]][partialTour[n]])).
                           add(a[partialTour[n]][1])).
                           compareTo(costOfBestTourSoFar) < 0))
         {// better tour found
            for (int j = 1; j <= n; j++)
               bestTourSoFar[j] = partialTour[j];
            costOfBestTourSoFar = ((Operable) costOfPartialTour.
                             add(a[partialTour[n - 1]][partialTour[n]])).
                             add(a[partialTour[n]][1]);
         }
      }
      else
      {// try out subtrees
         for (int j = currentLevel; j <= n; j++)
            // is move to subtree labeled partialTour[j] possible?
            if (a[partialTour[currentLevel - 1]][partialTour[j]] != null &&
                (costOfBestTourSoFar == null ||
                 ((Operable) remainingSum.add(costOfPartialTour.
                   add(a[partialTour[currentLevel - 1]][partialTour[j]]))).
                   compareTo(costOfBestTourSoFar) < 0))
            {// search this subtree
               MyMath.swap(partialTour, currentLevel, j);
               costOfPartialTour.increment(a[partialTour[currentLevel - 1]]
                                            [partialTour[currentLevel]]);
               remainingSum.decrement(minOut[partialTour[currentLevel]]);
               rTSP(currentLevel + 1);
               costOfPartialTour.decrement(a[partialTour[currentLevel - 1]]
                                            [partialTour[currentLevel]]);
               remainingSum.increment(minOut[partialTour[currentLevel]]);
               MyMath.swap(partialTour, currentLevel, j);
            }
      }
   }

   public static void main(String [] args)
   {

      // define a MyInputStream object to input from System.in
      MyInputStream keyboard = new MyInputStream();
      // input a test graph
      System.out.println("Enter number of vertices and edges");
      int n = keyboard.readInteger();
      int e = keyboard.readInteger();
      BTSalespersonWithStrongerCondition g
               = new BTSalespersonWithStrongerCondition(n);
      for (int i = 1; i <= e; i++)
      {
         System.out.println("enter weighted edge " + i);
         int u = keyboard.readInteger();
         int v = keyboard.readInteger();
         int w = keyboard.readInteger();
         g.putEdge(new WeightedEdge(u, v, new MyInteger(w)));
      }

      System.out.println("\nThe weighted digraph is");
      g.output();

      // test btSalesperson2
      int [] p = new int [n + 1];
      System.out.print("\nThe length of the shortest tour is ");
      System.out.println(g.btSalesperson3(p, new MyInteger(0)));
      System.out.print("The shortest tour is ");
      for (int i = 1; i <= n; i++)
         System.out.print(p[i] + " ");
      System.out.println();
   }
}
