
/** iterator for binary search trees, gets iterator in ascending order */

package dataStructures;

import java.util.*;

public class BinarySearchTreeWithIterator extends BinarySearchTree
{

   /** create and return an iterator */
   public Iterator iterator()
      {return new BinaryTreeIterator();}

   /** binary tree iterator */
   private class BinaryTreeIterator implements Iterator
   {
      // data members
      private BinaryTreeNode nextNode;  // next node in inorder
      private ArrayStack stack = new ArrayStack(10);
   
      // constructor
      public BinaryTreeIterator()
      {// position currentNode at first node in inorder
      
         if (root == null)
            // empty tree
            nextNode = null; 
      
         // moe to leftmost node
         BinaryTreeNode currentNode = root;
         while (currentNode.leftChild != null)
         {
            stack.push(currentNode);
            currentNode = currentNode.leftChild;
         }
         nextNode = currentNode;
      }
   
      // methods
      /** @return true iff tree has more iterator */
      public boolean hasNext()
         {return nextNode != null;}
   
      /** @return next element in tree
        * @throws NoSuchElementException
        * when there is no next element */
      public Object next()
      {
         if (nextNode != null)
         {
            Object obj = ((Data) nextNode.element).element;
            
            // set nextNode to next node in inorder
            BinaryTreeNode currentNode = nextNode.rightChild;
            if (currentNode != null)
            {// find leftmost element in right subtree
               // make as many leftChild moves as possible
               while (currentNode.leftChild != null)
               {
                  stack.push(currentNode);
                  currentNode = currentNode.leftChild;
               }
               nextNode = currentNode;
            }
            else
            {// no right subtree to traverse
               // back up to last node
               try
               {
                  nextNode = (BinaryTreeNode) stack.pop();
               }
               catch (Exception e)
               {// no node to back up to
                  nextNode = null;
               }
            }
            return obj;
         }
         else throw new NoSuchElementException("No next element");
      }

      /** unsupported method */
      public void remove()
      {
         throw new UnsupportedOperationException
                   ("remove not supported");
      }   
   }

   /** test binary search tree iterator */
   public static void main(String [] args)
   {
      int n = 30;

      // create a binary search tree with at most n iterator
      Random rand = new Random();
      BinarySearchTreeWithIterator tree =
                      new BinarySearchTreeWithIterator();
      for (int i = 0; i < n; i++)
      {
         int k = Math.abs(rand.nextInt()) % n;
         // duplicate keys are overwritten
         tree.put(new Integer(k), new Integer(k));
      }

      // output in ascending order
      System.out.println("The elements are");
      tree.ascend();
      System.out.println();

      // output in ascending order using an iterator
      System.out.println("The elements in iterator order are");
      Iterator it = tree.iterator();
      while (it.hasNext())
         System.out.print(it.next() + " ");
      System.out.println();
   }
}
