
/** split a binary search tree, recursive code */

package dataStructures;

public class BinarySearchTreeWithRecursiveSplit extends BinarySearchTree
{
   /** split the binary search tree around the key theKey
     * all elements > theKey end up in greaterThan
     * all elements < theKey end up in lessThan
     * @return the element (if any) with key theKey */
   private static BinaryTreeNode smallTreeRoot, // root of tree with
                                                // elements smaller than theKey
                                 bigTreeRoot;
   private static Object splitElement;
   private static Comparable splitKey;
   public Object split(Comparable theKey, BinarySearchTree lessThan,
                                          BinarySearchTree greaterThan)
   {
      splitElement = null;
      splitKey = theKey;
      split(root); // do the split recursively;
      lessThan.root = smallTreeRoot;
      greaterThan.root = bigTreeRoot;
      return splitElement;
   }

   /** split the binary tree whose root is theRoot */
   private static void split(BinaryTreeNode theRoot)
   {
      if (theRoot == null)
      {// split an empty tree
         smallTreeRoot = null;
         bigTreeRoot = null;
         return;
       }

       // split a non empty tree
       if (splitKey.compareTo(((Data) theRoot.element).key) < 0)
       {// root, its right subtree and part of its left subtree are in big tree
        // part of the left subtree is the small tree
          split(theRoot.leftChild);
          theRoot.leftChild = bigTreeRoot;
          bigTreeRoot = theRoot;
          return;
       }

       if (splitKey.compareTo(((Data) theRoot.element).key) > 0)
       {// root, its left subtree and part of its right subtree are in
        // the small tree
        // part of the right subtree is the big tree
          split(theRoot.rightChild);
          theRoot.rightChild = smallTreeRoot;
          smallTreeRoot = theRoot;
          return;
       }

       // splitKey equals key in root
       splitElement = theRoot.element;
       smallTreeRoot = theRoot.leftChild;
       bigTreeRoot = theRoot.rightChild;
       return;
   }

   // test the split method
   public static void main(String [] args)
   {
      BinarySearchTreeWithRecursiveSplit y =
                  new BinarySearchTreeWithRecursiveSplit();
      int [] splitKey = {1, 2, 6, 12, 15, 18, 20, 30};   

      for (int i = 0; i < splitKey.length; i++)
      {
         // insert a few elements
         y.put(new Integer(2), new Character('a'));
         y.put(new Integer(18), new Character('h'));
         y.put(new Integer(10), new Character('e'));
         y.put(new Integer(6), new Character('c'));
         y.put(new Integer(14), new Character('g'));
         y.put(new Integer(12), new Character('f'));
         y.put(new Integer(4), new Character('b'));
         y.put(new Integer(20), new Character('i'));
         y.put(new Integer(8), new Character('d'));
         System.out.println("Elements in ascending order are");
         y.ascend();
         System.out.println();
   
         BinarySearchTree small = new BinarySearchTree();
         BinarySearchTree large = new BinarySearchTree();
         System.out.println("The split key is " + splitKey[i]);
         System.out.println("The split element is " +
                             y.split(new Integer(splitKey[i]), small, large));
         System.out.println("The smaller elements are");
         small.ascend();
         System.out.println();
         System.out.println("The larger elements are");
         large.ascend();
         System.out.println();
         System.out.println();
      }
   }
}
