
/** split a binary search tree */

package dataStructures;

public class BinarySearchTreeWithSplit extends BinarySearchTree
{
   /** split the binary search tree around the key theKey
     * all elements > theKey end up in greaterThan
     * all elements < theKey end up in lessThan
     * @return the element (if any) with key theKey */
   public Object split(Comparable theKey, BinarySearchTree lessThan,
                                          BinarySearchTree greaterThan)
   {
      // initialize lessThan and greaterThan to have just a root
      // node which will later be removed
      lessThan.root = new BinaryTreeNode();
      greaterThan.root = new BinaryTreeNode();

      BinaryTreeNode lt = lessThan.root;       // lowest node in lessThan
      BinaryTreeNode gt = greaterThan.root;    // lowest node in greaterThan
      BinaryTreeNode currentNode = root;       // current node in this
      root = null;       // tree will be empty following the split

      // search for theKey and construct lessThan and greaterThan
      // during the search
      while (currentNode != null)
         if (theKey.compareTo(((Data) currentNode.element).key) > 0)
         {// attach currentNode and its left subtree to lt as the right
          // subtree and move into the right subtree of currentNode
            lt.rightChild = currentNode;
            lt = currentNode;
            currentNode = currentNode.rightChild;
         }
         else
            if (theKey.compareTo(((Data) currentNode.element).key) < 0)
            {// attach currentNode and its right subtree to gt as the left
             // subtree and move into the left subtree of currentNode
               gt.leftChild = currentNode;
               gt = currentNode;
               currentNode = currentNode.leftChild;
            }
            else
            {// found a match
               // take care of left and right subtrees of currentNode
               lt.rightChild = currentNode.leftChild;
               gt.leftChild = currentNode.rightChild;

               // eliminate fictitious root nodes
               lessThan.root = lessThan.root.rightChild;
               greaterThan.root = greaterThan.root.leftChild;

               return currentNode.element;
           }   
 
      // no matching element
      // set null pointers in lt and gt
      lt.rightChild = null;
      gt.leftChild = null;

      // eliminate fictitious root nodes
      lessThan.root = lessThan.root.rightChild;
      greaterThan.root = greaterThan.root.leftChild;

      return null;
   }

   // test the split method
   public static void main(String [] args)
   {
      BinarySearchTreeWithSplit y = new BinarySearchTreeWithSplit();
      int [] splitKey = {1, 2, 6, 12, 15, 18, 20, 30};   

      for (int i = 0; i < splitKey.length; i++)
      {
         // insert a few elements
         y.put(new Integer(2), new Character('a'));
         y.put(new Integer(18), new Character('h'));
         y.put(new Integer(10), new Character('e'));
         y.put(new Integer(6), new Character('c'));
         y.put(new Integer(14), new Character('g'));
         y.put(new Integer(12), new Character('f'));
         y.put(new Integer(4), new Character('b'));
         y.put(new Integer(20), new Character('i'));
         y.put(new Integer(8), new Character('d'));
         System.out.println("Elements in ascending order are");
         y.ascend();
         System.out.println();
   
         BinarySearchTree small = new BinarySearchTree();
         BinarySearchTree large = new BinarySearchTree();
         System.out.println("The split key is " + splitKey[i]);
         System.out.println("The split element is " +
                             y.split(new Integer(splitKey[i]), small, large));
         System.out.println("The smaller elements are");
         small.ascend();
         System.out.println();
         System.out.println("The larger elements are");
         large.ascend();
         System.out.println();
         System.out.println();
      }
   }
}
