
/** C-matrix mapped into a one-dimensional array */

package dataStructures;

import utilities.*;
import wrappers.*;

public class CMatrix
{
   // data members
   int rows;            // matrix dimension
   Object zero;         // zero element
   Object [] element;   // element array

   // constructor
   public CMatrix(int theRows, Object theZero)
   {
      // validate theRows
      if (theRows < 1)
         throw new IllegalArgumentException
                   ("number of rows must be > 0");
   
      // create and initialize the matrix
      rows = theRows;
      zero = theZero;
      element = new Object [3 * rows - 2];
      for (int i = 0; i < 3 * rows - 2; i++)
         element[i] = zero;
   }
   
   // methods
   /** @throws IndexOutOfBoundsException when i < 1
     * or j < 1 or i > rows or j > rows */
   void checkIndex(int i, int j)
   {
      if (i < 1 || j < 1 || i > rows || j > rows)
         throw new IndexOutOfBoundsException
                   ("i = " + i + " j = " + j +
                    " rows = " + rows + " cols = " + rows);
   }

   /** @return the element this(i,j)
     * @throws IndexOutOfBoundsException when i or j invalid */
   public Object get(int i, int j)
   {
      checkIndex(i, j);

      // determine element to return
      if (i == 1)
         // element from first row
         return element[j - 1];
      if (i == rows)
         // element from last row
         return element[rows + j - 1];
      if (j == 1)
         // rest of first column
         return element[2 * rows + i - 2]; 

      // outside the C
      return zero;
   }
   
   /** set this(i,j) = newValue
     * @throws IndexOutOfBoundsException when i or j invalid */
   public void set(int i, int j, Object newValue)
   {
      checkIndex(i, j);

      // store newValue
      if (i == 1)
         // first row element
         element[j - 1] = newValue;
      else
         if (i == rows)
            // element is on last row
            element[rows + j - 1] = newValue;
         else
            // rest of first column
            if (j == 1)
               element[2 * rows + i - 2] = newValue;
            else
               if (!((Zero) newValue).equalsZero())
                      throw new IllegalArgumentException
                          ("elements not on tridiagonal must be zero");
   }
   
   
   /** test program */
   public static void main(String [] args)
   {
      // create the matrix
      CMatrix x = new CMatrix(8, new MyInteger(0));

      // set a few elements
      x.set(1, 1, new MyInteger(22));
      x.set(1, 5, new MyInteger(44));
      x.set(8, 3, new MyInteger(0));
      x.set(6, 1, new MyInteger(55));

      // retrieve a few elements
      System.out.println(x.get(7, 8));
      System.out.println(x.get(5, 5));
      System.out.println(x.get(1, 1));
      System.out.println(x.get(6, 1));
      System.out.println(x.get(8, 3));
   }
}
