/** meld two chains */

package dataStructures;

public class ChainWithMeld extends Chain
{
   /** make this the result of melding the lists a and b,
     * i.e., take elements alternately from a and b
     * following the meld, both a and b are empty */
   public void meld(ChainWithMeld a, ChainWithMeld b)
   {
      // special cases
      if (a.size == 0)
      {// a is empty
         // take all nodes from b
         firstNode = b.firstNode;
         size = b.size;

         // make b empty
         b.firstNode = null;
         b.size = 0;

         return;
      }
      
      if (b.size == 0)
      {// b is empty
         // take all nodes from a
         firstNode = a.firstNode;
         size = a.size;

         // make a empty
         a.firstNode = null;
         a.size = 0;

         return;
      }
   
      // neither a nor b is empty
      ChainNode ca = a.firstNode,     // current node in a
                cb = b.firstNode,     // current node in b
                lt;                   // last node in this

      // meld first nodes of a and b
      firstNode = ca;
      ca = ca.next;
      firstNode.next = cb;
      lt = cb;
      cb = cb.next;
   
      // meld until one of a and b becomes empty
      while (ca != null && cb != null)
      {
         // first from a
         lt.next = ca;
         lt = ca;
         ca = ca.next;

         // then from b
         lt.next = cb;
         lt = cb; 
         cb = cb.next;
      }
      
      // append remaining elements
      // at most one of a and b can be nonempty now
      if (ca != null)
         lt.next = ca;
      else
         lt.next = cb;

      // compute size of result
      size = a.size + b.size;

      // make a and b empty
      a.firstNode = b.firstNode = null;
      a.size = b.size = 0;
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 10, m = 5;
      ChainWithMeld x = new ChainWithMeld();
      ChainWithMeld y = new ChainWithMeld();
      ChainWithMeld z = new ChainWithMeld();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println("Size of melded list is " + z.size());
      System.out.println("Size of input lists is " + x.size() 
                          + " " + y.size());
      System.out.println();
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println();
      
      // try it again
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + y);
      System.out.println("Second list is " + x);
      z.meld(y, x);
      System.out.println("Melded list is " + z);
   }
}
