/** merge two sorted lists */

package dataStructures;

public class ChainWithMerge extends Chain
{
   /** make this the result of merging the sorted lists a and b 
     * following the merge, both a and b are empty */
   public void merge(ChainWithMerge a, ChainWithMerge b)
   {
      // special cases
      if (a.size == 0)
      {// a is empty
         firstNode = b.firstNode;
         size = b.size;

         // make b empty
         b.firstNode = null;
         b.size = 0;

         return;
      }

      if (b.size == 0)
      {// b is empty
         firstNode = a.firstNode;
         size = a.size;

         // make a empty
         a.firstNode = null;
         a.size = 0;

         return;
      }
   
      // neither a nor b is empty
      ChainNode ca = a.firstNode,       // pointer to current node in a
                cb = b.firstNode,       // pointer to current node in b
                lt;                     // pointer to last node in this
      // merge first node for this
      if (((Comparable) ca.element).compareTo(cb.element) <= 0)
      {// first node of this is from a
         firstNode = lt = ca;
         ca = ca.next;
      }
      else
      {// first node of this is from b
         firstNode = lt = cb;
         cb = cb.next;
      }
   
      // merge from a and b until one becomes empty
      while (ca != null && cb != null)
      {
         if (((Comparable) ca.element).compareTo(cb.element) <= 0)
         {// a element goes first
            lt.next = ca;
            lt = ca;
            ca = ca.next;
         }
         else
         {// b element is smaller
            lt.next = cb;
            lt = cb;
            cb = cb.next;
         }
      }

      // append the rest, exactly one of a and b is nonempty now
      if (cb == null)
         lt.next = ca;
      else
         lt.next = cb;

      size = a.size + b.size;

      // make a and b empty
      a.firstNode = b.firstNode = null;
      a.size = b.size = 0;

      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 8, m = 5;
      ChainWithMerge x = new ChainWithMerge();
      ChainWithMerge y = new ChainWithMerge();
      ChainWithMerge z = new ChainWithMerge();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i) - 1));
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.merge(x, y);
      System.out.println("Merged list is " + z);
      System.out.println();

      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.merge(y, x);
      System.out.println("Merged list is " + z);
      System.out.println();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i) - 1));
      
      System.out.println("First list is " + y);
      System.out.println("Second list is " + x);
      z.merge(y, x);
      System.out.println("Merged list is " + z);
      System.out.println();
   }
}
