/** merge sort method to sort a chain */

package dataStructures;

public class ChainWithMergeSort extends Chain
{
   // constructors
   public ChainWithMergeSort(int initialCapacity)
      {super(initialCapacity);}

   public ChainWithMergeSort()
      {super();}

   /** in-place merge of the chains beginning at nodes a and b
      @return first node of resulting chain */
   static ChainNode merge(ChainNode a, ChainNode  b)
   {// merge the chains beginning at nodes a and b.
      // special cases when one or both is/are empty
      if (a == null)
         {return b;}
      if (b == null)
         {return a;}
   
      // neither a nor b is empty
      ChainNode last,   // pointer to last node in result
                first;  // pointer to first node in result
      // get first node of result
      if (((Comparable) a.element).compareTo(b.element) <= 0)
      {
         first = last = a;
         a = a.next;
      }
      else
      {
         first = last = b;
         b = b.next;
      }
   
      // attach remaining nodes in sorted order
      while (a != null && b != null)
         if (((Comparable) a.element).compareTo(b.element) <= 0)
         {// attach a to end of result
            last.next = a;
            last = a;
            a = a.next;
         }
         else
         {// attach b to end of result
            last.next = b;
            last = b;
            b = b.next;
         }
        
      // one of a and b has been exhausted
      // attach nodes from the nonempty chain
      if (a != null)
         // a is nonempty
         last.next = a;
      else
         // b is nonempty
         last.next = b;
   
      return first;
   }
   
   /** sort the chain using the merge sort method */
   public void mergeSort()
      {firstNode = mergeSort(firstNode, size);}

   /** recursively sort the chain that begins at node a and
     * has length l, use the merge sort method
     * @return first node of sorted chain */
   static ChainNode mergeSort(ChainNode a, int l)
   {
      if (l < 2) return a;  // chain of size 1 is always sorted
   
      // split chain into two approximately equal length chains
      // that begin at b and c respectively
      ChainNode b = a,
                c;
      // find mid point of length l chain
      for (int i = 1; i < l/2; i++)
         b = b.next;
      c = b.next;                     // start of second chain
      b.next = null;                  // close first chain

      a = mergeSort(a, l / 2);        // sort first chain
      c = mergeSort(c, l - l / 2);    // sort second chain
      return merge(a, c);             // merge sorted halves
   }

   /** test program */
   public static void main(String [] args)
   {
      ChainWithMergeSort x = new ChainWithMergeSort();
      x.add(0, new Integer(3));
      x.add(1, new Integer(2));
      x.add(2, new Integer(4));
      x.add(3, new Integer(1));
      x.add(4, new Integer(6));
      x.add(5, new Integer(9));
      x.add(6, new Integer(8));
      x.add(7, new Integer(7));
      x.add(8, new Integer(5));
      x.add(9, new Integer(0));
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.mergeSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);
   }
}
