/** natural merge sort method to sort a chain */

package dataStructures;

public class ChainWithNaturalMergeSort extends Chain
{
   // define a queue used for the start positions of sorted segments
   static ArrayQueue queue = new ArrayQueue(10);
   static int numberOfSegments;

   // constructors
   public ChainWithNaturalMergeSort(int initialCapacity)
      {super(initialCapacity);}

   public ChainWithNaturalMergeSort()
      {super();}
   
   /** sort the chain using the natural merge sort method */
   public void naturalMergeSort()
   {
      if (size <= 1)
         // already sorted
         return;

      // find the natural segments and save their start positions in a queue
      queue.put(firstNode);  // first segment starts at firstNode
      numberOfSegments = 1;
      ChainNode previousNode = firstNode;
      ChainNode currentNode = firstNode.next;
      while (currentNode != null)
      {
         if (((Comparable) previousNode.element).
                           compareTo(currentNode.element) > 0)
         {// a new segment starts at currentNode
            // close previous segment's chain
            previousNode.next = null;

            numberOfSegments++;
            queue.put(currentNode);
         }

         // advance to next node on chain
         previousNode = currentNode;
         currentNode = currentNode.next;
      }

      while (numberOfSegments > 1)
         // numberOfSegments is updated by naturalMergePass
         naturalMergePass();

      // set firstNode for sorted chain
      firstNode = (ChainNode) queue.getFrontElement();
   }

   /** merge pairs of adjacent segments from the queue */
   public static void naturalMergePass()
   {
      for (int i = 1; i < numberOfSegments; i += 2)
      {// merge two segments
         // get the two segments that are to be merged
         ChainNode startA = (ChainNode) queue.remove();
         ChainNode startB = (ChainNode) queue.remove();

         // merge and put resulting segment on queue
         queue.put(merge(startA, startB));
      }
   
      // one segment may remain
      if (numberOfSegments % 2 == 1)
         // move the segment from the front to the rear
         queue.put(queue.remove());

      // update number of segments
      if (numberOfSegments % 2 == 1)
         numberOfSegments = numberOfSegments / 2 + 1;
      else
         numberOfSegments /= 2;
   }

   /** in-place merge of the chains beginning at nodes a and b
      @return first node of resulting chain */
   static ChainNode merge(ChainNode a, ChainNode  b)
   {// merge the chains beginning at nodes a and b.
      // special cases when one or both is/are empty
      if (a == null)
         return b;
      if (b == null)
         return a;
   
      // neither a nor b is empty
      ChainNode last,   // pointer to last node in result
                first;  // pointer to first node in result
      // get first node of result
      if (((Comparable) a.element).compareTo(b.element) <= 0)
      {
         first = last = a;
         a = a.next;
      }
      else
      {
         first = last = b;
         b = b.next;
      }
   
      // attach remaining nodes in sorted order
      while (a != null && b != null)
         if (((Comparable) a.element).compareTo(b.element) <= 0)
         {// attach a to end of result
            last.next = a;
            last = a;
            a = a.next;
         }
         else
         {// attach b to end of result
            last.next = b;
            last = b;
            b = b.next;
         }
        
      // one of a and b has been exhausted
      // attach nodes from the nonempty chain
      if (a != null)
         // a is nonempty
         last.next = a;
      else
         // b is nonempty
         last.next = b;
   
      return first;
   }

   /** test program */
   public static void main(String [] args)
   {
      ChainWithNaturalMergeSort x = new ChainWithNaturalMergeSort();
      x.add(0, new Integer(3));
      x.add(1, new Integer(2));
      x.add(2, new Integer(4));
      x.add(3, new Integer(1));
      x.add(4, new Integer(6));
      x.add(5, new Integer(9));
      x.add(6, new Integer(8));
      x.add(7, new Integer(7));
      x.add(8, new Integer(5));
      x.add(9, new Integer(0));
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.naturalMergeSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);
   }
}
