
/** extension of Chain to include the method removeRange */

package dataStructures;

public class ChainWithRemoveRange extends Chain
{
   /** remove the elements whose index is in the range
     * start through finish-1 */
   public void removeRange(int start, int finish)
   {
      // validate start and finish indexes
      if (start < 0)
         start = 0;
      if (finish > size)
         finish = size;
      if (start >= finish)
         // no elements to remove
         return;

      if (start == 0)
      {// remove first finish - 1 nodes
          // find first node that remains
          for (int i = 0; i < finish; i++)
             firstNode = firstNode.next;
      }
      else
      {// remove from interior
         // find node just before first removed node
         ChainNode justBeforeNode = firstNode;
         for (int i = 1; i < start; i++)
            justBeforeNode = justBeforeNode.next;

         // find last node to remove
         ChainNode lastNodeToRemove = justBeforeNode;
         for (int i = start; i < finish; i++)
            lastNodeToRemove = lastNodeToRemove.next;
         
         // remove the nodes
         justBeforeNode.next = lastNodeToRemove.next;
      }
      size -= finish - start;
   }
     

   /** test program */
   public static void main(String [] args)
   {
      ChainWithRemoveRange x = new ChainWithRemoveRange();

      x.add(0, new Integer(0));
      x.add(1, new Integer(1));
      x.add(2, new Integer(2));
      x.add(3, new Integer(3));
      x.add(4, new Integer(4));
      x.add(5, new Integer(5));
      x.add(6, new Integer(6));
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      // remove from interior
      x.removeRange(1,4);
      System.out.println("Removed elements 1 through 3");
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      // remove one node from front
      x.removeRange(0,1);
      System.out.println("Removed elements 0 through 0");
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      // remove all nodes
      x.removeRange(0,10);
      System.out.println("Removed elements 0 through 9");
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      x.add(0, new Integer(0));
      x.add(1, new Integer(1));
      x.add(2, new Integer(2));
      x.add(3, new Integer(3));
      x.add(4, new Integer(4));
      x.add(5, new Integer(5));
      x.add(6, new Integer(6));
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      // remove one interior node
      x.removeRange(2,3);
      System.out.println("Removed element 2");
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);

      // remove all nodes beginning at node 2
      x.removeRange(2,10);
      System.out.println("Removed element 2 and all after it");
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
   }
}
