

/** extension of Chain to include setSize */

package dataStructures;

import utilities.*;

public class ChainWithSetSize extends Chain
{
   // constructors
   public ChainWithSetSize(int initialCapacity)
      {super(initialCapacity);}

   public ChainWithSetSize()
   {// use default capacity of 10
      this(10);
   }

   /** make size = theSize
     * @throws IllegalArgumentException when
     * theSize < 0 */
   public void setSize(int theSize)
   {
      if (theSize < 1)
         throw new IllegalArgumentException
                   ("theSize must be >= 0, it is " + theSize);

      // handle by cases
      if (theSize == 0)
         // resulting list is empty
         firstNode = null;
      else
         if (theSize <  size)
         { // eliminate some nodes
            // find last node to retain
            ChainNode lastNodeToKeep = firstNode;
            for (int i = 1; i < theSize; i++)
               lastNodeToKeep = lastNodeToKeep.next;
   
            // eliminate the rest
            lastNodeToKeep.next = null;
         }
         else
         {// add null elements
            int numberToAdd = theSize - size;
            if (numberToAdd == 0)
               return;

            // find last node of chain, would be easy if we were
            // asked to extend ExtendedChain instead of Chain
            ChainNode currentNode = firstNode;
            for (int i = 1; i < size; i++)
               currentNode = currentNode.next;

            // link first node into chain
            ChainNode newNode = new ChainNode(null, null);
            if (currentNode == null)
               // list is presently empty
               firstNode = newNode;
            else
               currentNode.next = newNode;
            currentNode = newNode;

            // now currentNode cannot be null, add in remaining nodes
            for (int i = 1; i < numberToAdd; i++)
            {
                currentNode.next = new ChainNode(null, null);
                currentNode = currentNode.next;
            }
         }
      size = theSize;
   }

   /** test program */
   public static void main(String [] args)
   {
      ChainWithSetSize x = new ChainWithSetSize(1);

      x.add(0, new Integer(0));
      x.add(1, new Integer(1));
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
      System.out.println();

      x.setSize(6);
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
      System.out.println();

      x.setSize(3);
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
      System.out.println();
   }
}
