/** split a chain into two by changing the next pointer in each node
  * currentNode to currentNode.next.next */

package dataStructures;

public class ChainWithSplit2 extends Chain
{
   /** split this into lists a and b, this becomes empty following the split */
   public void split(ChainWithSplit2 a, ChainWithSplit2 b)
   {
      // handle empty chain as a special case
      if (size == 0)
      {// both a and b are empty
         a.firstNode = b.firstNode = null;
         a.size = b.size = 0;
         return;
      }

      // length is >= 1, assign a and b their first nodes
      ChainNode currentNode = firstNode,
                nextNode = currentNode.next;   // node next to currentNode
      a.firstNode = currentNode;
      b.firstNode = nextNode;
   
      // change currentNode.next to nextNode.next in all but the
      // last node of the chain
      while (nextNode != null)
      {
         currentNode.next = nextNode.next;

         // move to next node
         currentNode = nextNode;
         nextNode = nextNode.next;
      }
   
      // end the chain whose last node is currentNode
      currentNode.next = null; 

      // set size values for a and b
      b.size = size / 2;
      a.size = size - b.size;
   
      // this is now empty
      firstNode = null;
      size = 0;
   
      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      ChainWithSplit2 x =new ChainWithSplit2();
      ChainWithSplit2 y =new ChainWithSplit2();
      ChainWithSplit2 z =new ChainWithSplit2();
      
      //  try list sizes 0 through 7
      for (int n = 0; n < 8; n++)
      {
         // put elements in x
         for (int i = 0; i < n; i++)
            x.add(i, new Integer(i));
         
         System.out.println("The list is " + x);
         x.split(y, z);
         System.out.println("The split lists are " + y + "\nand " + z);
         System.out.println();
      }
   }
}
