
/** chained hash table with a put with visit method */

package dataStructures;

import java.lang.reflect.*;
import utilities.*;

public class ChainedHashTableWithVisit extends ChainedHashTable
{
   // constructor
   public ChainedHashTableWithVisit(int theDivisor)
      {super(theDivisor);}

   /** insert an element with the specified key
     * visit old element if there is already an
     * element with the given key */
   public void put(Object theKey, Object theElement, Method theVisit)
   {
      int b = theKey.hashCode() % divisor;  // home bucket

      // search home bucket chain
      HashNode currentNode = table[b],
               previousNode = null;

      while (currentNode != null && currentNode.key.compareTo(theKey) < 0)
      {// move to next node on home bucket chain
         previousNode = currentNode;
         currentNode = currentNode.next;
      }
   
      // check if element with theKey already in table
      if (currentNode != null && currentNode.key.equals(theKey))
      {// visit element with same key
         Object [] visitArgs = new Object [1];
         visitArgs[0] = currentNode.element;
         try {theVisit.invoke(null, visitArgs);}
         catch (Exception e)
            {System.out.println(e);}
         return;
      }
   
      // insert theElement after previousNode and before currentNode
      HashNode newNode = new HashNode((Comparable) theKey, theElement);
      newNode.next = currentNode;
      if (previousNode == null)
         // insert at front
         table[b] = newNode;
      else
         // not at front
         previousNode.next = newNode;

      size++;
   }
}
