/** merge two sorted lists represented as circular lists */

package dataStructures;

public class CircularListWithMerge extends CircularList
{
   /** make this the result of merging the sorted lists a and b 
     * following the merge, both a and b are empty */
   public void merge(CircularListWithMerge a, CircularListWithMerge b)
   {
      // special cases
      if (a.size == 0)
      {// a is empty
         lastNode = b.lastNode;
         size = b.size;

         // make b empty
         b.clear();

         return;
      }

      if (b.size == 0)
      {// b is empty
         lastNode = a.lastNode;
         size = a.size;

         // make a empty
         a.clear();

         return;
      }
   
      // neither a nor b is empty
      ChainNode ca = a.lastNode.next,       // pointer to current node in a
                cb = b.lastNode.next,       // pointer to current node in b
                firstNode;                  // pointer to first node in this

      // convert a and b to chains
      a.lastNode.next = b.lastNode.next = null;

      // merge first node for this
      if (((Comparable) ca.element).compareTo(cb.element) <= 0)
      {// first node of this is from a
         lastNode = firstNode = ca;
         ca = ca.next;
      }
      else
      {// first node of this is from b
         lastNode = firstNode = cb;
         cb = cb.next;
      }
   
      // merge from a and b until one becomes empty
      while (ca != null && cb != null)
         if (((Comparable) ca.element).compareTo(cb.element) <= 0)
         {// a element goes first
            lastNode.next = ca;
            lastNode = ca;
            ca = ca.next;
         }
         else
         {// b element is smaller
            lastNode.next = cb;
            lastNode = cb;
            cb = cb.next;
         }

      // append the rest, exactly one of a and b is nonempty now
      if (ca == null)
      {
         lastNode.next = ca;
         lastNode = a.lastNode;
      }  
      else
      {
         lastNode.next = cb;
         lastNode = b.lastNode;
      }

      // make into a circular list
      lastNode.next = firstNode;
      size = a.size + b.size;

      // make a and b empty
      a.clear();
      b.clear();

      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 8, m = 5;
      CircularListWithMerge x = 
          new CircularListWithMerge();
      CircularListWithMerge y = 
          new CircularListWithMerge();
      CircularListWithMerge z = 
          new CircularListWithMerge();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i) - 1));
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.merge(x, y);
      System.out.println("Merged list is " + z);
      System.out.println();

      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.merge(y, x);
      System.out.println("Merged list is " + z);
      System.out.println();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i) - 1));
      
      System.out.println("First list is " + y);
      System.out.println("Second list is " + x);
      z.merge(y, x);
      System.out.println("Merged list is " + z);
      System.out.println();
   }
}
