/** split a circular list into two */

package dataStructures;

public class CircularListWithSplit extends CircularList
{
   /** split this into lists a and b, this becomes empty following the split */
   public void split(CircularListWithSplit a, CircularListWithSplit b)
   {
      // handle this.size < 2 as special cases
      if (size < 2)
      {
         if (size == 0)
         {// both a and b are empty
            a.clear();
            b.clear();
            return;
         }

         // size is 1
         // a gets the element, this and b become empty
         a.lastNode = lastNode;
         a.size = 1;

         // make b and this empty
         b.clear();
         clear();

         return;
      }
   
      // length is >= 2, assign a and b their first nodes
      ChainNode firstA = lastNode.next;
      ChainNode firstB = firstA.next;

      // assign remaining nodes
      ChainNode ct = firstB.next;          // cursor for this
      a.lastNode = firstA;
      b.lastNode = firstB;
      while (ct != firstA)
      {
         // last give a node to a
         a.lastNode.next = ct;
         a.lastNode = ct;
         ct = ct.next;
         if (ct == firstA)
            break;

         // now give a node to b
         b.lastNode.next = ct;
         b.lastNode = ct;
         ct = ct.next;
      }
   
      // close the circles
      a.lastNode.next = firstA;
      b.lastNode.next = firstB;

      // set size values for a and b
      b.size = size / 2;
      a.size = size - b.size;
   
      // this is now empty
      clear();
   
      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      CircularListWithSplit x =new CircularListWithSplit();
      CircularListWithSplit y =new CircularListWithSplit();
      CircularListWithSplit z =new CircularListWithSplit();
      
      //  try list sizes 0 through 7
      for (int n = 0; n < 8; n++)
      {
         // put elements in x
         for (int i = 0; i < n; i++)
            x.add(i, new Integer(i));
         
         System.out.println("The list is " + x);
         x.split(y, z);
         System.out.println("The split lists are " + y + "\nand " + z);
         System.out.println();
      }
   }
}
