/** meld two doubly linked circular lists */

package dataStructures;

public class DoubleCircularListWithMeld extends DoubleCircularList
{
   /** make this the result of melding the lists a and b,
     * i.e., take elements alternately from a and b
     * following the meld, both a and b are empty */
   public void meld(DoubleCircularListWithMeld a, DoubleCircularListWithMeld b)
   {
      // special cases
      if (a.size == 0)
      {// a is empty, answer is b
         lastNode = b.lastNode;
         size = b.size;

         // make b empty
         b.clear();

         return;
      }
      
      if (b.size == 0)
      {// b is empty, answer is a
         lastNode = a.lastNode;
         size = a.size;

         // make a empty
         a.clear();

         return;
      }
   
      // neither a nor b is empty
      DoubleNode ca = a.lastNode.next,     // current node in a
                 cb = b.lastNode.next;     // current node in b

      // make a and b noncircular
      a.lastNode.next = null;
      b.lastNode.next = null;

      // meld first nodes of a and b
      DoubleNode firstNode = ca;
      ca = ca.next;
      firstNode.next = cb;
      lastNode = cb;
      lastNode.previous = firstNode;
      cb = cb.next;
   
      // meld until one of a and b is exhausted
      while (ca != null && cb != null)
      {
         // first from a
         lastNode.next = ca;
         ca.previous = lastNode;
         lastNode = ca;
         ca = ca.next;

         // then from b
         lastNode.next = cb;
         cb.previous = lastNode;
         lastNode = cb; 
         cb = cb.next;
      }
      
      // append remaining elements
      // at most one of a and b can be nonempty now
      if (ca != null)
      {
         lastNode.next = ca;
         ca.previous = lastNode;
         lastNode = a.lastNode;
      }
      if (cb != null)
      {
         lastNode.next = cb;
         cb.previous = lastNode;
         lastNode = b.lastNode;
      }

      // compute size of result
      size = a.size + b.size;

      // close the circle
      lastNode.next = firstNode;
      firstNode.previous = lastNode;

      // make a and b empty
      a.clear();
      b.clear();
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 10, m = 5;
      DoubleCircularListWithMeld x = new DoubleCircularListWithMeld();
      DoubleCircularListWithMeld y = new DoubleCircularListWithMeld();
      DoubleCircularListWithMeld z = new DoubleCircularListWithMeld();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println("Size of melded list is " + z.size());
      System.out.println("Size of input lists is " + x.size() 
                          + " " + y.size());
      System.out.println();
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println();
      
      // try it again
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + y);
      System.out.println("Second list is " + x);
      z.meld(y, x);
      System.out.println("Melded list is " + z);
   }
}
