/** split a doubly linked circular list into two */

package dataStructures;

public class DoubleCircularListWithSplit extends DoubleCircularList
{
   /** split this into lists a and b,
     * this becomes empty following the split */
   public void split(DoubleCircularListWithSplit a,
                     DoubleCircularListWithSplit b)
   {
      // handle this.size < 2 as special cases
      if (size < 2)
      {
         if (size == 0)
         {// both a and b are empty
            a.clear();
            b.clear();
            return;
         }

         // size = 1
         // a gets the element, this and b become empty
         a.lastNode = lastNode;
         a.size = 1;
         clear();
         b.clear();
         return;
      }

      // length is >= 2, assign a and b their first nodes
      DoubleNode firstA = a.lastNode = lastNode.next;
      DoubleNode firstB = b.lastNode = firstA.next;
   
      // assign remaining nodes
      lastNode.next = null;          // make this noncircular
      DoubleNode ct = firstB.next;   // cursor for this
      while (ct != null)
      {
         // first give a node to a
         a.lastNode.next = ct;
         ct.previous = a.lastNode;
         a.lastNode = ct;
         ct = ct.next;
         if (ct == null)
            break;

         // now give a node to b
         b.lastNode.next = ct;
         ct.previous = b.lastNode;
         b.lastNode = ct;
         ct = ct.next;
      }
   
      // close the circles
      a.lastNode.next = firstA;
      b.lastNode.next = firstB;

      // set size values for a and b
      b.size = size / 2;
      a.size = size - b.size;
   
      // this is now empty
      clear();
   
      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      DoubleCircularListWithSplit x =new DoubleCircularListWithSplit();
      DoubleCircularListWithSplit y =new DoubleCircularListWithSplit();
      DoubleCircularListWithSplit z =new DoubleCircularListWithSplit();
      
      //  try list sizes 0 through 7
      for (int n = 0; n < 8; n++)
      {
         // put elements in x
         for (int i = 0; i < n; i++)
            x.add(i, new Integer(i));
         
         System.out.println("The list is " + x);
         x.split(y, z);
         System.out.println("The split lists are " + y + "\nand " + z);
         System.out.println();
      }
   }
}
