/** split a doubly linked list into two */

package dataStructures;

public class DoublyLinkedListWithSplit extends DoublyLinkedList
{
   /** split this into lists a and b,
     * this becomes empty following the split */
   public void split(DoublyLinkedListWithSplit a,
                     DoublyLinkedListWithSplit b)
   {
      // handle this.size < 2 as special cases
      if (size < 2)
      {
         if (size == 0)
         {// both a and b are empty
            a.clear();
            b.clear();
            return;
         }

         // size = 1
         // a gets the element, this and b become empty
         a.firstNode = a.lastNode = firstNode;
         a.size = 1;
         clear();
         b.clear();
         return;
      }

      // length is >= 2, assign a and b their first nodes
      a.firstNode = firstNode;
      a.lastNode = a.firstNode;
      b.firstNode = firstNode.next;
      b.lastNode = b.firstNode;
      b.firstNode.previous = null;
   
      // assign remaining nodes
      DoubleNode ct = b.firstNode.next;   // cursor for this
      while (ct != null)
      {
         // first give a node to a
         a.lastNode.next = ct;
         ct.previous = a.lastNode;
         a.lastNode = ct;
         ct = ct.next;
         if (ct == null)
            break;

         // now give a node to b
         b.lastNode.next = ct;
         ct.previous = b.lastNode;
         b.lastNode = ct;
         ct = ct.next;
      }
   
      // set null pointers in last nodes
      a.lastNode.next = b.lastNode.next = null;

      // set size values for a and b
      b.size = size / 2;
      a.size = size - b.size;
   
      // this is now empty
      clear();
   
      return;
   }

   /** test program */
   public static void main(String [] args)
   {
      DoublyLinkedListWithSplit x =new DoublyLinkedListWithSplit();
      DoublyLinkedListWithSplit y =new DoublyLinkedListWithSplit();
      DoublyLinkedListWithSplit z =new DoublyLinkedListWithSplit();
      
      //  try list sizes 0 through 7
      for (int n = 0; n < 8; n++)
      {
         // put elements in x
         for (int i = 0; i < n; i++)
            x.add(i, new Integer(i));
         
         System.out.println("The list is " + x);
         x.split(y, z);
         System.out.println("The split lists are " + y + "\nand " + z);
         System.out.println();
      }
   }
}
