/** extension of diagonal matrix, methods to input, output,
  * add, subtract, and transpose included */

package dataStructures;

import java.lang.reflect.*;
import utilities.*;
import wrappers.*;

public class ExtendedDiagonalMatrix extends DiagonalMatrix
{
   // constructor
   public ExtendedDiagonalMatrix(int theRows, Object theZero)
      {super(theRows, theZero);}
   
  // methods
  /** create string suitable for output */
  public String toString()
  {
     StringBuffer s = new StringBuffer(); 
  
     s.append("Main diagonal is \n");
     for (int i = 0; i < rows; i++)
        s.append(element[i] + "  ");
     s.append("\n");

     // create equivalent String
     return new String(s);
  }
  
  /** input a diagonal matrix from the given input stream */
  public void input(Object theZero, MyInputStream inStream)
  {
     Method inputMethod;
     Object [] inputMethodArgs = {inStream};
     Class [] parameterTypes = {inStream.getClass()};
     zero = theZero;
     try
     {
        // get the proper method to be used to read in the values
        inputMethod = theZero.getClass().
                         getMethod("input", parameterTypes);
  
        // input number of rows in the matrix
        System.out.println("Enter number of rows");
        rows = inStream.readInteger();
        // validate input
        if (rows < 1)
           throw new IllegalArgumentException
                     ("number of rows must be > 0");
     
        // create the element array
        element = new Object [rows];
  
        // input elements
        System.out.println("Enter main diagonal");
        for (int i = 0; i < rows; i++)
           element[i] = inputMethod.invoke(null, inputMethodArgs);
     }
     catch (Exception e)
     {
        System.out.println(e);
        throw new IllegalArgumentException
              ("must have the static method input() defined");
     }
  }

  /** @return this + b */
  public ExtendedDiagonalMatrix add(ExtendedDiagonalMatrix b)
  {
     if (rows != b.rows)
        throw new IllegalArgumentException
                  ("Matrices must have same dimensions");
  
     // create result array w
     ExtendedDiagonalMatrix w = new ExtendedDiagonalMatrix(rows, zero);
     for (int i = 0; i < rows; i++)
         w.element[i] = ((Computable) element[i]).add(b.element[i]);
  
     return w;
  }
  
  /** @return this - b */
  public ExtendedDiagonalMatrix subtract(ExtendedDiagonalMatrix b)
  {
     if (rows != b.rows)
        throw new IllegalArgumentException
                  ("Matrices must have same dimensions");
  
     // create result array w
     ExtendedDiagonalMatrix w = new ExtendedDiagonalMatrix(rows, zero);
     for (int i = 0; i < rows; i++)
         w.element[i] = ((Computable) element[i]).subtract(b.element[i]);
  
     return w;
  }
  
  /** @return this * b */
  public ExtendedDiagonalMatrix multiply(ExtendedDiagonalMatrix b)
  {
     if (rows != b.rows)
        throw new IllegalArgumentException
                  ("Matrices must have same dimensions");
  
     // create result array w
     ExtendedDiagonalMatrix w = new ExtendedDiagonalMatrix(rows, zero);
     for (int i = 0; i < rows; i++)
         w.element[i] = ((Computable) element[i]).multiply(b.element[i]);
  
     return w;
  }
  
  /** @return the transpose of this */
  public ExtendedDiagonalMatrix transpose()
  {
  
     // create result array w
     ExtendedDiagonalMatrix w = new ExtendedDiagonalMatrix(rows, zero);
  
     // copy diagonal of this to diagonal of w
     for (int i = 0; i < rows; i++)
        {w.element[i] = element[i];}
  
     return w;
  }
   
   /** test program */
   public static void main(String [] args)
   {
      // create the matrix
      ExtendedDiagonalMatrix x =
              new ExtendedDiagonalMatrix(1, new MyInteger(0));
      ExtendedDiagonalMatrix y =
              new ExtendedDiagonalMatrix(1, new MyInteger(0));

      MyInputStream keyboard = new MyInputStream();
      // test input
      x.input(new MyInteger(0), keyboard);
      y.input(new MyInteger(0), keyboard);

      // test output
      System.out.println(x);
      System.out.println();
      System.out.println(y);
      System.out.println();

      // test add, subtract, and transpose
      System.out.println("The sum of the matrices is\n" + x.add(y));
      System.out.println();
      System.out.println("Their difference is\n" + x.subtract(y));
      System.out.println();
      System.out.println("Their product is\n" + x.multiply(y));
      System.out.println();
      System.out.println("The transpose of the first matrix is\n"
                         + x.transpose());
   }
}
