
/** extension of LowerTriangularMatrix, methods to input, output,
  * add, subtract, and transpose included */

package dataStructures;

import java.lang.reflect.*;
import utilities.*;
import wrappers.*;

public class ExtendedLowerTriangularMatrix2 extends LowerTriangularMatrix
{
   // constructor
   public ExtendedLowerTriangularMatrix2(int theRows, Object theZero)
      {super(theRows, theZero);}
   
  // methods
  /** create string suitable for output */
  public String toString()
  {
     StringBuffer s = new StringBuffer(); 
  
     s.append("The elements of the lower triangle, in row-major order are\n");
     for (int i = 0; i < element.length; i++)
        s.append(element[i] + "  ");
     s.append("\n");
  
     // create equivalent String
     return new String(s);
  }
  
  /** input a lower triangular matrix from the given input stream */
  public void input(Object theZero, MyInputStream inStream)
  {
     Method inputMethod;
     Object [] inputMethodArgs = {inStream};
     Class [] parameterTypes = {inStream.getClass()};
     zero = theZero;
     try
     {
        // get the proper method to be used to read in the values
        inputMethod = theZero.getClass().
                         getMethod("input", parameterTypes);
  
        // input number of rows in the matrix
        System.out.println("Enter number of rows");
        rows = inStream.readInteger();
        // validate input
        if (rows < 1)
           throw new IllegalArgumentException
                     ("number of rows must be > 0");
     
        // create the element array
        int numOfElements = rows * (rows + 1) / 2;
        element = new Object [numOfElements];
  
        // input elements
        System.out.println("Enter the " + numOfElements +
                   " elements of the lower triangle in row-major order");
        for (int i = 0; i < numOfElements; i++)
           element[i] = inputMethod.invoke(null, inputMethodArgs);
     }
     catch (Exception e)
     {
        System.out.println(e);
        throw new IllegalArgumentException
         ("matrix element type "
          + "must have the static method input() defined");
     }
  }

  /** @return this + b */
  public ExtendedLowerTriangularMatrix2 add(ExtendedLowerTriangularMatrix2 b)
  {
     if (rows != b.rows)
        throw new IllegalArgumentException
                  ("Matrices must have same dimensions");
  
     // create result array w
     ExtendedLowerTriangularMatrix2 w =
                  new ExtendedLowerTriangularMatrix2(rows, zero);
     for (int i = 0; i < element.length; i++)
         w.element[i] = ((Computable) element[i]).add(b.element[i]);
  
     return w;
  }
  
  /** @return this - b */
  public ExtendedLowerTriangularMatrix2
         subtract(ExtendedLowerTriangularMatrix2 b)
  {
     if (rows != b.rows)
        throw new IllegalArgumentException
                  ("Matrices must have same dimensions");
  
     // create result array w
     ExtendedLowerTriangularMatrix2 w =
                  new ExtendedLowerTriangularMatrix2(rows, zero);
     for (int i = 0; i < element.length; i++)
         w.element[i] = ((Computable) element[i]).subtract(b.element[i]);
  
     return w;
  }
   
   /** test program */
   public static void main(String [] args)
   {
      // create the matrix
      ExtendedLowerTriangularMatrix2 x =
              new ExtendedLowerTriangularMatrix2(1, new MyInteger(0));
      ExtendedLowerTriangularMatrix2 y =
              new ExtendedLowerTriangularMatrix2(1, new MyInteger(0));

      MyInputStream keyboard = new MyInputStream();
      // test input
      x.input(new MyInteger(0), keyboard);
      y.input(new MyInteger(0), keyboard);

      // test output
      System.out.println(x);
      System.out.println();
      System.out.println(y);
      System.out.println();

      // test add and subtract
      System.out.println("The sum of the matrices is\n" + x.add(y));
      System.out.println();
      System.out.println("Their difference is\n" + x.subtract(y));
      System.out.println();
   }
}
