/** meld two formula-based linear lists represented using Equation 3.2 */

package dataStructures;

import utilities.*;

public class FormulaBasedLinearList8WithMeld
       extends FormulaBasedLinearList8
{
   /** make this the result of melding the lists a and b,
     * i.e., take elements alternately from a and b */
   public void meld(FormulaBasedLinearList8WithMeld a,
                    FormulaBasedLinearList8WithMeld b)
   {
      // compute size of result
      int aSize = a.size();
      int bSize = b.size();
      int size = aSize + bSize;

      // make sure this.element is big enough for result
      if (size > element.length)
         // get bigger array
         element = new Object [size];

      int ct = 0;                            // cursor for this
      int ca = a.first;                      // cursor for a
      int cb = b.first;                      // cursor for b
      int s = Math.min(aSize, bSize);        // smaller of aSize and bSize
      
      // copy s elements from a and b to this
      for (int i = 0; i < s; i++)
      {
         // copy elements
         element[ct++] = a.element[ca];
         element[ct++] = b.element[cb];

         // advance cursors for a and b
         ca = (ca + 1) % a.element.length;
         cb = (cb + 1) % b.element.length;
      }
      
      // take care of left overs (if any)
      for (int i = s; i < aSize; i++)
      {
         element[ct++] = a.element[ca];
         ca = (ca + 1) % a.element.length;
      }

      for (int i = s; i < bSize; i++)
      {
         element[ct++] = b.element[cb];
         cb = (cb + 1) % b.element.length;
      }

      // set data members of this
      first = 0;
      last = size - 1;
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 10, m = 5;
      FormulaBasedLinearList8WithMeld x = new FormulaBasedLinearList8WithMeld();
      FormulaBasedLinearList8WithMeld y = new FormulaBasedLinearList8WithMeld();
      FormulaBasedLinearList8WithMeld z = new FormulaBasedLinearList8WithMeld();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.insertElementAt(new Integer(2*(n - i)), 0);
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.insertElementAt(new Integer(2*(n - i)), 0);
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      z.meld(y, x);
      System.out.println("Melded list is " + z);
   }
}
