/** split a formula-based linear list into two
  * Equation 3.2 is used */

package dataStructures;

import utilities.*;

public class FormulaBasedLinearList8WithSplit
       extends FormulaBasedLinearList8
{
   /** split this into lists a and b */
   public void split(FormulaBasedLinearList8WithSplit a,
                     FormulaBasedLinearList8WithSplit b)
   {
      // compute size of each split list, note that bSize()
      // is 1 less than aSize() when this.size() is odd
      int bSize = size() / 2;
      int aSize = size() - bSize;

      // make sure each has enough space
      if (a.element.length < aSize)
         a.element = new Object [aSize];
      if (b.element.length < bSize)
         b.element = new Object [bSize];
      
      int ct = first;         // cursor for this
      for (int i = 0; i < bSize; i++)
      {
         a.element[i] = element[ct];
         ct = (ct + 1) % element.length;
         b.element[i] = element[ct];
         ct = (ct + 1) % element.length;
      }
      
      // is an element left?
      if (aSize > bSize)
         a.element[aSize - 1] = element[ct];

      // set data members of a and b
      a.first = 0;
      a.last = aSize - 1;
      b.first = 0;
      b.last = bSize - 1;
   }

   /** test program */
   public static void main(String [] args)
   {
      FormulaBasedLinearList8WithSplit x =
                  new FormulaBasedLinearList8WithSplit();
      FormulaBasedLinearList8WithSplit y =
                  new FormulaBasedLinearList8WithSplit();
      FormulaBasedLinearList8WithSplit z =
                  new FormulaBasedLinearList8WithSplit();
      
      //  first try a list with even size
      int n = 10;
      // put elements in x
      for (int i = 0; i < n; i++)
         x.insertElementAt(new Integer(i), i);
      
      System.out.println("The list is " + x);
      x.split(y, z);
      System.out.println("The split lists are " + y + "\nand " + z);
      System.out.println();
      
      //  now try a list with odd size
      // put one more element into x
      x.insertElementAt(new Integer(n), n);
      
      System.out.println("The list is " + x);
      x.split(y, z);
      System.out.println("The split lists are " + y + "\nand " + z);
      System.out.println();
   }
}
