
/** formula-based implementation of LinearList that decreases the size
  * of the element array when occupancy drops below 25% */

package dataStructures;

import java.util.*;
import wrappers.*;
import utilities.*;

public class FormulaBasedLinearListWithDecreaseSize
                          extends FormulaBasedLinearList
{
   // additional data member
   protected int initialCapacity;       // initial size of element array

   // constructors
   /** create a list with initial capacity initialCapacity */
   public FormulaBasedLinearListWithDecreaseSize(int initialCapacity)
   {
      super(initialCapacity);
      this.initialCapacity = initialCapacity;
   }

   /** create a list with initial capacity 10 */
   public FormulaBasedLinearListWithDecreaseSize()
   {// use default capacity of 10
      this(10);
   }

   /** overrides FormulaBasedLinearList.removeElementAt */
   public void removeElementAt(int index)
   {
      super.removeElementAt(index);
  
      // decrease size of element array if occupancy is below 25%
      if (size < element.length / 4 && element.length / 2 >= initialCapacity)
         // halve the size of the array
         element = ChangeArraySize.changeSize1D
                      (element, size, element.length / 2);
   }

   /** test program */
   public static void main(String [] args)
   {
      LinearList x = new FormulaBasedLinearListWithDecreaseSize(2);
      int n = 10;

      // insert n elements
      for (int i = 0; i < n; i++)
      {
         x.insertElementAt(new MyInteger(i), 0);
         System.out.println("The capacity is " +
                ((FormulaBasedLinearList) x).element.length);
      }
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);


      // remove n-1 elements
      for (int i = 0; i < n - 1; i++)
      {
         System.out.println("Removing element " + x.elementAt(0));
         x.removeElementAt(0);
         System.out.println("The capacity is " +
                ((FormulaBasedLinearList) x).element.length);
      }
      System.out.println("List size is " + x.size());
      System.out.println("The list is " + x);
   }
}
