/** meld two formula-based linear lists */

package dataStructures;

import utilities.*;

public class FormulaBasedLinearListWithMeld
       extends FormulaBasedLinearList
{
   /** make this the result of melding the lists a and b,
     * i.e., take elements alternately from a and b */
   public void meld(FormulaBasedLinearListWithMeld a,
                     FormulaBasedLinearListWithMeld b)
   {
      // compute size of result
      size = a.size + b.size;

      // make sure this.element is big enough for result
      if (size > element.length)
         // get bigger array
         element = new Object [size];

      int ct = 0;                        // cursor for this
      int s = Math.min(a.size, b.size);  // smaller of a.size and b.size
      
      // copy s elements from a and b to this
      for (int i = 0; i < s; i++)
      {
         element[ct++] = a.element[i];
         element[ct++] = b.element[i];
      }
      
      // take care of left overs (if any)
      for (int i = s; i < b.size; i++)
         element[ct++] = b.element[i];

      for (int i = s; i < a.size; i++)
         element[ct++] = a.element[i];
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 10, m = 5;
      FormulaBasedLinearListWithMeld x = new FormulaBasedLinearListWithMeld();
      FormulaBasedLinearListWithMeld y = new FormulaBasedLinearListWithMeld();
      FormulaBasedLinearListWithMeld z = new FormulaBasedLinearListWithMeld();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.insertElementAt(new Integer(2*(n - i)), 0);
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.insertElementAt(new Integer(2*(n - i)), 0);
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      z.meld(y, x);
      System.out.println("Melded list is " + z);
   }
}
