/** extension of HashTable to include remove method */

package dataStructures;

import java.util.*;

public class HashTableWithRemove extends HashTable
{
   public HashTableWithRemove(int theDivisor)
   {
      super(theDivisor);
   }

   /** search an open addressed hash table for an element with key theKey
     * @return location of matching element if found, otherwise return
     * last location examined
     * this method is identical to HashTable.search and is duplicated here
     * because HashTable.search is private */
   private int search(Object theKey)
   {
      int i = Math.abs(theKey.hashCode()) % divisor;  // home bucket
      int j = i;    // start at home bucket
      do
      {
         if (table[j] == null || table[j].key.equals(theKey))
            return j;
         j = (j + 1) % divisor;  // next bucket
      } while (j != i);          // returned to home bucket?
   
      return j;  // table full
   }

   /** @return matching element and remove it
     * @return null if no matching element */
   public Object remove(Object theKey)
   {
      // search for matching element
      int b = search(theKey);

      // check if we have a match
      if (table[b] == null || !table[b].key.equals(theKey))
         // no element with theKey in table
         return null;
   
      // we have a match, save element to be removed
      Object elementToReturn = table[b].element;
      table[b] = null;

      // move elements as needed
      int nextBucket = (b + 1) % divisor;
      while (table[nextBucket] != null)
      {
         int home = Math.abs(table[nextBucket].key.hashCode()) % divisor;
               // home bucket
         if (!(
               (b < home && home <= nextBucket) ||
               (nextBucket < b && b < home) ||
               (home <= nextBucket && nextBucket < b)))
         {// move element in table[nextBucket]
            table[b] = table[nextBucket];
            table[nextBucket] = null;
            b = nextBucket;  // empty bucket
         }
         nextBucket = (nextBucket + 1) % divisor;
      }
      size--;
      return elementToReturn;
   }

   /** test method */
   public static void main (String [] args)
   {
      HashTableWithRemove h = new HashTableWithRemove(19);
      h.put(new Integer(80), new Integer(80));
      h.put(new Integer(40), new Integer(40));
      h.put(new Integer(65), new Integer(65));
      h.put(new Integer(58), new Integer(58));
      h.put(new Integer(24), new Integer(24));
      h.put(new Integer(2), new Integer(2));
      h.put(new Integer(13), new Integer(13));
      h.put(new Integer(46), new Integer(46));
      h.put(new Integer(16), new Integer(16));
      h.put(new Integer(7), new Integer(7));
      h.put(new Integer(21), new Integer(21));
      h.put(new Integer(99), new Integer(99));
      h.put(new Integer(17), new Integer(17));
      h.put(new Integer(35), new Integer(35));
      h.put(new Integer(54), new Integer(54));
      h.output();
      System.out.println();
      h.remove(new Integer(7));
      h.remove(new Integer(40));
      h.remove(new Integer(16));
      h.output();
   }
}
