/** meld two circular lists with header nodes */

package dataStructures;

public class HeadCircularListWithMeld extends HeadCircularList
{
   /** make this the result of melding the lists a and b,
     * i.e., take elements alternately from a and b
     * following the meld, both a and b are empty */
   public void meld(HeadCircularListWithMeld a, HeadCircularListWithMeld b)
   {
      ChainNode ca = a.headerNode.next,     // current node in a
                cb = b.headerNode.next,     // current node in b
      lastNode = headerNode;                // last node in this

      // meld until one of a and b is exhausted
      while (ca != a.headerNode && cb != b.headerNode)
      {
         // first from a
         lastNode.next = ca;
         lastNode = ca;
         ca = ca.next;

         // then from b
         lastNode.next = cb;
         lastNode = cb; 
         cb = cb.next;
      }
      
      // append remaining elements
      // at most one of a and b can be nonempty now
      if (ca != a.headerNode)
      {
         lastNode.next = ca;
         lastNode = a.lastNode;
      }
      if (cb != b.headerNode)
      {
         lastNode.next = cb;
         lastNode = b.lastNode;
      }

      // complete the circle
      lastNode.next = headerNode;

      // compute size of result
      size = a.size + b.size;

      // make a and b empty
      a.clear();
      b.clear();
   }

   /** test program */
   public static void main(String [] args)
   {
      int n = 10, m = 5;
      HeadCircularListWithMeld x = new HeadCircularListWithMeld();
      HeadCircularListWithMeld y = new HeadCircularListWithMeld();
      HeadCircularListWithMeld z = new HeadCircularListWithMeld();
      
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println("Size of melded list is " + z.size());
      System.out.println("Size of input lists is " + x.size() 
                          + " " + y.size());
      System.out.println();
      
      System.out.println("First list is " + x);
      System.out.println("Second list is " + y);
      z.meld(x, y);
      System.out.println("Melded list is " + z);
      System.out.println();
      
      // try it again
      //  put elements in x
      for (int i = 0; i < n; i++)
         x.add(0, new Integer(2 * (n - i)));
      
      //  put elements in y
      for (int i = 0; i < m; i++)
         y.add(0, new Integer(2 * (n - i)));
      
      System.out.println("First list is " + y);
      System.out.println("Second list is " + x);
      z.meld(y, x);
      System.out.println("Melded list is " + z);
   }
}
