
/** ideal hashing */

package dataStructures;

public class IdealHashTable
{
   // data members
   private Object [] table;      // hash table array
   private int size;             // number of elements in table
   private int maxKey;           // max permissible key

   // constructor
   public IdealHashTable(int theMaxKey)
   {
      maxKey = theMaxKey;

      // allocate hash table array
      table = new Object [maxKey + 1];
   }
   
   // instance methods
   /** @return true iff the hash table is empty */
   public boolean isEmpty()
       {return size == 0;}

   /** @return current number of elements in the hash table */
   public int size()
      {return size;}

   /** @return element with specified key
     * @return null if no matching element */
   public Object get(int theKey)
   {
      if (theKey < 0 || theKey > maxKey)
         return null;
      else
         return table[theKey];
   }
   
   /** insert an element with the specified key
     * overwrite old element if there is already an
     * element with the given key
     * @return old element (if any) with key = theKey */
   public Object put(int theKey, Object theElement)
   {
      if (theKey < 0 || theKey > maxKey)
         // invalid key value
         throw new IllegalArgumentException
                   ("key must be between 0 and " + maxKey);

      // insert into the table
      if (table[theKey] == null && theElement != null)
         size++;
      Object elementToReturn = table[theKey];
      table[theKey] = theElement;
      return elementToReturn;
   }

   /** @return matching element and remove it
     * @return null if no matching element */
   public Object remove(int theKey)
   {
      if (theKey < 0 || theKey > maxKey)
         return null;
      else
      {
         Object x = table[theKey];
         if (x != null)
            size--;
         table[theKey] = null;
         return x;
      }
   }
   
   /** output the hash table */
   public void output()
   {
      for (int i = 0; i <= maxKey; i++)
         System.out.println(table[i]);
      System.out.println("Table size is " + size);
   }
   
   /** test method */
   public static void main(String [] args)
   {
      IdealHashTable h = new IdealHashTable(11);
      h.put(8, new Integer(80));
      h.put(4, new Integer(40));
      h.put(6, new Integer(65));
      h.output();
      System.out.println();
      h.put(5, new Integer(58));
      h.put(2, new Integer(24));
      h.output();
      System.out.println();
      h.put(2, new Integer(2));
      h.put(1, new Integer(13));
      h.put(4, new Integer(46));
      h.put(1, new Integer(16));
      h.put(7, new Integer(7));
      h.put(2, new Integer(21));
      h.output();
      System.out.println();
   }
}
