
/** iterative postorder traversal of a linked binary tree */

package dataStructures;

public class IterativePostorderTraversal
{
   /** visit method that prints the element in the node */ 
   public static void visit(BinaryTreeNode t)
      {System.out.print(t.element + " ");}

   // define a class for use by postOrder
   static class StackElement
   {
      // data members
      BinaryTreeNode node;
      boolean leftChild;   // true iff we move to the left child of node

      // constructor
      public StackElement(BinaryTreeNode theNode, boolean theLeftChild)
      {
         node = theNode;
         leftChild = theLeftChild;
      }
   }
   
   
   /** postorder traversal */
   public static void postOrder(BinaryTreeNode t)
   {
      ArrayStack stack = new ArrayStack(10);
      BinaryTreeNode currentNode = t;
   
      while (true)
      {// traverse subtree rooted at currentNode in postorder
            
         // make as many leftChild moves as possible
         while (currentNode != null)
         {
            stack.push(new StackElement(currentNode, true));
            currentNode = currentNode.leftChild;
         }
   
         // back up the tree, visiting nodes whose right subtrees
         // we are returning from until we back up from a left
         // subtree or have no node to back up to
         while (true)
         {// back up one level
            StackElement se;
            try
            {
               se = (StackElement) stack.pop();
            }
            catch (Exception e)
            {// no node to back up to
               return;
            }

            currentNode = se.node;
            if (se.leftChild)
               // returning from a left subtree
               break;

            // returning to currentNode from its right subtree
            visit(currentNode);
         }
   
         // returned from left subtree of currentNode
         // put it back on the stack and go into its right subtree
         stack.push(new StackElement(currentNode, false));
         currentNode = currentNode.rightChild;
      }
   }

   /** test program */
   public static void main(String [] args)
   {
      BinaryTreeNode y = new BinaryTreeNode(new Integer(4));
      BinaryTreeNode z = new BinaryTreeNode(new Integer(5));
      BinaryTreeNode x = new BinaryTreeNode(new Integer(2), y, z);
      BinaryTreeNode w = new BinaryTreeNode(new Integer(7));
      BinaryTreeNode v = new BinaryTreeNode(new Integer(6), null, w);
      y = new BinaryTreeNode(new Integer(3), v, null);
      z = new BinaryTreeNode(new Integer(1), x, y);

      System.out.println("The elements in postorder are");
      postOrder(z);
      System.out.println();
   }
}
