/** linear list as an indexed binary tree */

package dataStructures;

import java.util.*;
import java.lang.reflect.*;

public class LinearListAsBinaryTree extends LinkedBinaryTree
                                    implements LinearList
{
   // top-level nested class
   static class IndexedElement
   {
      // data members
      Object element;        // element in node
      int leftSize;          // number of nodes in left subtree

      // constructors
      IndexedElement(int theLeftSize, Object theElement)
      {
         leftSize = theLeftSize;
         element = theElement;
      }

      IndexedElement(Object theElement)
         {element = theElement;}

      /** @return equivalent string suitable for output */
      public String toString()
         {return element.toString();}
   }

   // instance data member of LinearListAsBinaryTree
   int size;  // number of elements

   // class data member used by indexOf and search
   static Object searchElement;

   // class data member used by add
   static Random rand = new Random();

   // class data member used by toString
   static StringBuffer buffer;

   /** visit method that adds an element to a string buffer */
   public static void addToStringBuffer(BinaryTreeNode x)
      {buffer.append(x + ", ");}

   // define a Method variable for string conversion
   static Class [] paramType = {BinaryTreeNode.class};
                             // type of parameter for visit
   static Method theAddToStringBuffer;

   // initialize theAddToStringBuffer
   static
   { 
      try
      {
         Class lbt = LinearListAsBinaryTree.class;
         theAddToStringBuffer = lbt.getMethod("addToStringBuffer", paramType);
      }
      catch (Exception e) {}
         // exception not possible
   }

   // methods of LinearList
   /** @return true iff list is empty */
   public boolean isEmpty()
      {return size == 0;}

   /** @return current number of elements in list */
   public int size()
      {return size;}

   /** @throws IndexOutOfBoundsException when
     * index is not between 0 and size - 1 */
   void checkIndex(int index)
   {
      if (index < 0 || index >= size)
         throw new IndexOutOfBoundsException
             ("index = " + index + "  size = " + size);
   }

   /** @return element with specified index
     * @throws IndexOutOfBoundsException
     * when index is not between 0 and size - 1 */
   public Object get(int index)
   {
      checkIndex(index);

      BinaryTreeNode p = root;
      while (true)
      {
         IndexedElement q = (IndexedElement) p.element;
         if (index < q.leftSize)
            // desired element is in left subtree
            p = p.leftChild;
         else
            if (index > q.leftSize)
            {// desired element is in left subtree
      	        index -= q.leftSize + 1;
                p = p.rightChild;
            }
            else 
               // found desired element
               return q.element; 
      }
   }

   /** @return index of first occurrence of theElement,
     * return -1 if elem not in list */
   public int indexOf(Object theElement)
   {
      searchElement = theElement;
      return search(root);
   }

   /** recursively search the subtree with root x
     * @return index of searchElement
     * @return -1 if searchElement not in tree   */
   int search(BinaryTreeNode x)
   {
      if (x == null)
         // empty subtree
         return -1;

      // look in left subtree first
      int index = search(x.leftChild);
      if (index != -1)
         // found in left subtree
         return index;

      // compare with element in root
      if (((IndexedElement) x.element).element.equals(searchElement))
         // found the element
         return ((IndexedElement) x.element).leftSize;

      // search the right subtree
      index = search(x.rightChild);
      if (index != -1)
         // found in right subtree
         return index + ((IndexedElement) x.element).leftSize + 1;

      // not found at all
      return -1;
   }
   
   
   /** Remove the element with specified index.
     * All elements with higher index have their
     * index reduced by 1.
     * @throws IndexOutOfBoundsException
     * when index is not between 0 and size - 1
     * @return removed element */
   public Object remove(int index)
   {
      checkIndex(index);
   
      // find element to remove
      BinaryTreeNode pp = null;  // parent of p
      BinaryTreeNode p = root;

      // set p to point to index'th element
      while (((IndexedElement) p.element).leftSize != index)
      {
         pp = p;
         if (index < ((IndexedElement) p.element).leftSize)
         {// move into left subtree
            ((IndexedElement) p.element).leftSize--;
            p = p.leftChild;
         }
         else
         {// move into right subtree
            index -= ((IndexedElement) p.element).leftSize + 1;
            p = p.rightChild;
         }
      }
   
      Object elementToReturn = ((IndexedElement) p.element).element;

      // proceed to delete from tree
      if (p.leftChild != null && p.rightChild != null)
      {// p has two children
         // find element with max index in left subtree of p
         // first move into left subtree, then make as
         // many right child moves as possible
         int ls = ((IndexedElement) p.element).leftSize - 1; // save value
         BinaryTreeNode ps = p,  // parent of s
                        s = p.leftChild;
         while (s.rightChild != null)
         {
            ps = s;
            s = s.rightChild;
         }
   
         // move max index element to p
         p.element = s.element;
         ((IndexedElement) p.element).leftSize = ls;

         // set pp and p so that p is node to delete
         pp = ps;
         p = s;
      }
   
      // now p has at most one child
      // save pointer to single subtree of p in s
      BinaryTreeNode s;
      if (p.leftChild != null)
         s = p.leftChild;
      else
         s = p.rightChild;
   
      // attach s to pp unless p is root
      if (p == root)
         root = s;
      else
         if (p == pp.leftChild)
            pp.leftChild = s;
         else pp.rightChild = s;

      size--;
      return elementToReturn;
   }
   
   /** Insert an element with specified index.
     * All elements with equal or higher index
     * have their index increased by 1.
     * @throws IndexOutOfBoundsException
     * when index is not between 0 and size */
   public void add(int index, Object theElement)
   {
      if (index < 0 || index > size)
         // invalid list position
         throw new IndexOutOfBoundsException
             ("index = " + index + "  size = " + size);

      // set up a node for obj and update size
      BinaryTreeNode q = new BinaryTreeNode
                             (new IndexedElement(0, theElement));
      size++;
   
      // treat empty tree as a special case
      if (root == null)
      {// tree is empty, index must be zero
         root = q;
         return;
      }
   
      // also do index = 0 as a special case
      if (index == 0) // tree is not empty
         // decide whether to make q the root
         if (Math.abs(rand.nextInt()) % 2 == 0)
         {// make q the root
            // old tree becomes right subtree of q
            q.rightChild = root;
            root = q;
            return;
         }
         else
         {// insert q as leftmost node of tree
            // p will eventually point to leftmost node
            BinaryTreeNode p = root;
            while (p.leftChild != null)
            {
               ((IndexedElement) p.element).leftSize++;
               p = p.leftChild;
            }
   
            // q should be left child of p
            p.leftChild = q;
            ((IndexedElement) p.element).leftSize++;
            return;
         }
   
      // index != 0
      BinaryTreeNode p = root;
      BinaryTreeNode pp = null;  // parent of p
      // set p to point to the (index - 1)'th element
      index--;
      while (index != ((IndexedElement) p.element).leftSize)
      {
         pp = p;
         if (index < ((IndexedElement) p.element).leftSize)
         {// will add q into left subtree of p
            ((IndexedElement) p.element).leftSize++;
            p = p.leftChild;
         }
         else
         {// will put q in right subtree of p
            index -= ((IndexedElement) p.element).leftSize + 1;
            p = p.rightChild;
         }
      }
   
      // decide whether to make q the right
      // child of p, the parent of p, or to insert q as the leftmost
      // node in the right subtree
      switch (Math.abs(rand.nextInt()) % 3)
      {
         case 0: // make q the right child of p
            // attach q as the right child of p and make
            // p's current right subtree the right subtree of p
            q.rightChild = p.rightChild;
            p.rightChild = q;
            return;
         case 1: // make q the parent of p
            // p becomes left child of q and right subtree
            // of p becomes right subtree of q
            q.leftChild = p;
            q.rightChild = p.rightChild;
            p.rightChild = null;
            ((IndexedElement) q.element).leftSize
                       = ((IndexedElement) p.element).leftSize + 1;
            // attach q to pp unless p is old root
            if (pp != null) // p is not old root
               if (p == pp.leftChild)
                  // q becomes left child of pp
                  // leftSize of pp already incremented
                  // do not increment again
                  pp.leftChild = q;
               else // q becomes right child of pp
                    pp.rightChild = q;
            else  // p is old root
               root = q;
   
            return;
   
         case 2: // insert q as leftmost node in right subtree of p
            // move to leftmost node in right subtree
            if (p.rightChild == null)
            {// empty right subtree
               p.rightChild = q;
               return;
            }

            // first move into right subtree
            p = p.rightChild;
            // now move to leftmost node
            while (p.leftChild != null)
            {
               ((IndexedElement) p.element).leftSize++;
               p = p.leftChild;
            }
   
            // q should be left child of p
            p.leftChild = q;
            ((IndexedElement) p.element).leftSize++;
            return;
      }
   }


   /** convert to a string */
   public String toString()
   {
      if (size == 0)
         return "[]";

      buffer = new StringBuffer("["); 
      inOrder(theAddToStringBuffer);

      // remove last blank character
      buffer.setLength(buffer.length() - 1);

      // overwrite last ,
      buffer.setCharAt(buffer.length() - 1, ']');

      // create equivalent String
      return new String(buffer);
   }

   /** test program */
   public static void main(String [] args)
   {
      // test default constructor
      LinearListAsBinaryTree x = new LinearListAsBinaryTree();

      // test size
      System.out.println("Initial size is " + x.size());

      // test isEmpty
      if (x.isEmpty())
         System.out.println("The list is empty");
      else System.out.println("The list is not empty");

      // test insertElementAt
      x.add(0, new Integer(2));
      x.add(1, new Integer(6));
      x.add(0, new Integer(1));
      x.add(2, new Integer(4));
      System.out.println("List size is " + x.size());

      // test toString
      System.out.println("The list is " + x);

      // test indexOf
      int index = x.indexOf(new Integer(4));
      if (index < 0)
         System.out.println("4 not found");
      else System.out.println("The index of 4 is " + index);

      index = x.indexOf(new Integer(3));
      if (index < 0)
         System.out.println("3 not found");
      else System.out.println("The index of 3 is " + index);

      // test get
      System.out.println("Element at 0 is " + x.get(0));
      System.out.println("Element at 3 is " + x.get(3));

      // test remove
      System.out.print("Removed " + x.remove(1) + " ");
      System.out.println("The list is " + x);
      System.out.print("Removed " + x.remove(2) + " ");
      System.out.println("The list is " + x);

      if (x.isEmpty())
         System.out.println("The list is empty");
      else System.out.println("The list is not empty");

      System.out.println("List size is " + x.size());
   }
}
