/** linked binary trees with a inorder iterator */

package dataStructures;

import java.util.*;

public class LinkedBinaryTreeWithInorderIterator extends LinkedBinaryTree
{
   /** create and return an iterator */
   public Iterator iterator()
      {return new InorderIterator();}

   /** inorder iterator */
   private class InorderIterator implements Iterator
   {
      // data members
      private BinaryTreeNode nextNode;  // next node in inorder
      private ArrayStack stack = new ArrayStack(10);
   
      // constructor
      /** set nextNode to first node in inorder */
      public InorderIterator()
      {
         if (root == null)
            // empty tree
            nextNode = null; 
      
         // move to leftmost node
         BinaryTreeNode currentNode = root;
         while (currentNode.leftChild != null)
         {
            stack.push(currentNode);
            currentNode = currentNode.leftChild;
         }
         nextNode = currentNode;
      }
   
      // methods
      /** @return true iff tree has more iterator */
      public boolean hasNext()
         {return nextNode != null;}

      /** @return next element in inorder
        * @throws NoSuchElementException
        * when there is no next element */
      public Object next()
      {
         if (nextNode != null)
         {
            Object obj = nextNode.element;
            
            // set nextNode to next node in inorder
            BinaryTreeNode currentNode = nextNode.rightChild;
            if (currentNode != null)
            {// find leftmost element in right subtree
               // make as many leftChild moves as possible
               while (currentNode.leftChild != null)
               {
                  stack.push(currentNode);
                  currentNode = currentNode.leftChild;
               }
               nextNode = currentNode;
            }
            else
            {// no right subtree to traverse
               // back up to last node
               try
               {
                  nextNode = (BinaryTreeNode) stack.pop();
               }
               catch (Exception e)
               {// no node to back up to
                  nextNode = null;
               }
            }
            return obj;
         }
         else throw new NoSuchElementException("No next element");
      }

      /** unsupported method */
      public void remove()
      {
         throw new UnsupportedOperationException
                   ("remove not supported");
      }   
   }

   /** test program */
   public static void main(String [] args)
   {
      LinkedBinaryTreeWithInorderIterator
                       a = new LinkedBinaryTreeWithInorderIterator(),
                       w = new LinkedBinaryTreeWithInorderIterator(), 
                       x = new LinkedBinaryTreeWithInorderIterator(), 
                       y = new LinkedBinaryTreeWithInorderIterator(), 
                       z = new LinkedBinaryTreeWithInorderIterator();
      y.makeTree(new Integer(1), a, a);
      z.makeTree(new Integer(2), a, a);
      x.makeTree(new Integer(3), y, z);
      y.makeTree(new Integer(4), x, a);
      w.makeTree(new Integer(5), a, a);
      z.makeTree(new Integer(6), a, a);
      x.makeTree(new Integer(7), w, z);
      y.makeTree(new Integer(8), x, y);

      System.out.println("Inorder sequence is ");
      y.inOrderOutput();
      System.out.println();

      System.out.println("Inorder enumeration yields");
      Iterator iy = y.iterator();
      while (iy.hasNext())
         System.out.print(iy.next() + " ");
      System.out.println();
   }
}
