
/** extension of LinkedStack to include methods that
  * add and remove nodes rather than elements */

package dataStructures;

import java.util.EmptyStackException;

public class LinkedStackWithNodeOperations extends LinkedStack
{
   // constructors
   /** create an empty stack */
   public LinkedStackWithNodeOperations(int initialCapacity)
      {super(initialCapacity);}

   public LinkedStackWithNodeOperations()
      {this(0);}

   // additional methods
   /** @return top node of stack 
     * @throws EmptyStackException when the stack is empty */
   public ChainNode peekNode()
   {
      if (empty())
         throw new EmptyStackException();
      return topNode;
   }

   /** add theNode to the top of the stack */
   public void pushNode(ChainNode theNode)
   {
      theNode.next = topNode;
      topNode = theNode;
   }

   /** remove top node of stack and return it
     * @throws EmptyStackException when the stack is empty */
   public ChainNode popNode()
   {
      if (empty())
         throw new EmptyStackException();
      ChainNode nodeToReturn = topNode;
      topNode = topNode.next;
      return nodeToReturn;
   }
   
   /** test program */
   public static void main(String [] args)
   {  
      LinkedStackWithNodeOperations s = new LinkedStackWithNodeOperations();
      // add a few elements
      s.push(new Integer(1));
      s.push(new Integer(2));
      s.push(new Integer(3));
      s.push(new Integer(4));
      ChainNode y = new ChainNode(new Integer(5));
      s.pushNode(y);


      // delete all elements
      while (!s.empty())
      {
         System.out.println("Top element is " + s.peekNode().element);
         System.out.println("Removed the element " + s.popNode().element);
      }
   }  
}
