
/** single-source all-destinations algorithm using the
  * linked adjacency list representation of a weighted digraph */

package dataStructures;

import java.util.*;
import utilities.*;
import wrappers.*;

public class LinkedWDigraphWithShortestPaths extends LinkedWDigraph
{
   // constructors
   public LinkedWDigraphWithShortestPaths(int theVertices)
      {super(theVertices);}
   
   // default is a 0 vertex graph
   public LinkedWDigraphWithShortestPaths()
      {this(0);}


   /** find shortest paths from vertex s
     * @return shortest distances in d
     * @return predecessor information in p */
   public void shortestPaths(int s, Operable [] d, int [] p)
   {
      if (s < 1 || s > n)
         throw new IllegalArgumentException
               ("source vertex cannot be " + s);

      GraphChain l = new GraphChain(); // list of reachable vertices for
                                       // which paths have yet to be found
      // initialize d, p, and l
      // first set all d values to null and all p values to -1
      for (int i = 1; i <= n; i++)
      {
          d[i] = null;
          p[i] = -1;
      }
      // now change d and p values for vertices adjacent from s
      Iterator is = aList[s].iterator();
      while (is.hasNext())
      {
         WeightedEdgeNode wNode = (WeightedEdgeNode) is.next();
         d[wNode.vertex] = (Operable) wNode.weight;
         d[s] = (Operable) ((Operable) wNode.weight).zero();
         p[wNode.vertex] = s; 
         l.add(0, new EdgeNode(wNode.vertex));
      }
      p[s] = 0;  // source vertex has no predecessor
   
      // update d and p
      while (!l.isEmpty())
      {// more paths exist
         // find vertex v in l with least d
         Iterator il = l.iterator();
         int v = ((EdgeNode) il.next()).vertex;
         while (il.hasNext())
         {
            int w = ((EdgeNode) il.next()).vertex;
            if (d[w].compareTo(d[v]) < 0)
               v = w;
         }
   
         // next shortest path is to vertex v
         // delete v from l and update d
         l.removeElement(v);
         Iterator iv = aList[v].iterator();
         while (iv.hasNext())
         {
            WeightedEdgeNode wNode = (WeightedEdgeNode) iv.next();
            int j = wNode.vertex;
      	    if (p[j] == -1 || d[j].compareTo(d[v].add(wNode.weight)) > 0)
            {
               // d[j] decreases
               d[j] = (Operable) d[v].add(wNode.weight);
               // add j to l if not already in l
               if (p[j] == -1)
                  l.add(0, new EdgeNode(j));
               p[j] = v;
            }
         }
      }
   }

   /** test shortestPaths */
   public static void main(String [] args)
   {
      // define a MyInputStream object to input from System.in
      MyInputStream keyboard = new MyInputStream();

      // input a test graph
      System.out.println("Enter number of vertices and edges");
      int n = keyboard.readInteger();
      int e = keyboard.readInteger();
      LinkedWDigraphWithShortestPaths g =
                    new LinkedWDigraphWithShortestPaths(n);
      for (int i = 1; i <= e; i++)
      {
         System.out.println("enter weighted edge " + i);
         int u = keyboard.readInteger();
         int v = keyboard.readInteger();
         int w = keyboard.readInteger();
         g.putEdge(new WeightedEdge(u, v, new MyInteger(w)));
      }

      System.out.println("\nThe weighted digraph is");
      g.output();

      // test shortestPaths
      MyInteger [] dist = new MyInteger [n + 1];
      int [] p = new int [n + 1];
      g.shortestPaths(1, dist, p);
      System.out.println("\nThe dist and p values for the directed graph are");
      for (int i = 1; i <= n; i++)
         System.out.println(dist[i] + " " + p[i]);
   }
}
