/** max hblt trees with a level order iterator */

package dataStructures;

import java.util.*;

public class MaxHbltWithIterator extends MaxHblt
{
   /** create and return an iterator */
   public Iterator iterator()
      {return new HbltIterator();}

   /** level order iterator */
   private class HbltIterator implements Iterator
   {
      // data members
      private HbltNode nextNode;  // next node in level order
      private ArrayQueue queue = new ArrayQueue();
   
      // constructor
      /** set nextNode to first node in level order */
      public HbltIterator()
         {nextNode = root;}
   
      // methods
      /** @return true iff tree has more iterator */
      public boolean hasNext()
         {return nextNode != null;}

      /** @return next element in level order
        * @throws NoSuchElementException
        * when there is no next element */
      public Object next()
      {
         if (nextNode != null)
         {
            // save next level order element
            Object obj = nextNode.element;

            // put children on queue
            if (nextNode.leftChild != null) 
               queue.put(nextNode.leftChild);
            if (nextNode.rightChild != null)
               queue.put(nextNode.rightChild);
      
            // get next node to visit
            nextNode = (HbltNode) queue.remove();
            
            return obj;
         }
         else throw new NoSuchElementException("No next element");
      }

      /** unsupported method */
      public void remove()
      {
         throw new UnsupportedOperationException
                   ("remove not supported");
      }   
   }

   /** test program */
   public static void main(String [] args)
   {
      MaxHbltWithIterator h = new MaxHbltWithIterator();
      Comparable [] z = new Comparable [10];
      for (int i = 1; i < 10; i++)
         z[i] = new Integer(i);
      h.initialize(z, 9);

      System.out.println("Level order iteration yields");
      Iterator ih = h.iterator();
      while (ih.hasNext())
         System.out.print(ih.next() + " ");
      System.out.println();
   }
}
