   
/** max heap extended to include the method changeMax */

package dataStructures;

public class MaxHeapWithChangeMax extends MaxHeap
{
   // constructors
   /** create a heap with the given initial capacity */
   public MaxHeapWithChangeMax(int initialCapacity)
      {super(initialCapacity);}
   
   /** create a heap with initial capacity 10 */
   public MaxHeapWithChangeMax()
      {super(10);}

   /** change the max element to theElement */
   public void changeMax(Comparable theElement)
   {
      if (size == 0)
      {// heap is empty, make a one element heap
         if (heap.length < 2)
            // allocate larger array
            heap = new Comparable [2];
         heap[1] = theElement;
         size = 1;
      }
         
      // nonempty heap, replace heap[1]
      Comparable y = theElement;      // last element
   
      // find place for y starting at root
      int i = 1,  // current node of heap
          ci = 2; // child of i
      while (ci <= size)
      {
         // heap[ci] should be larger child of i
         if (ci < size &&
             heap[ci].compareTo(heap[ci + 1]) < 0)
            ci++;
   
         // can we put y in heap[i]?
         if (y.compareTo(heap[ci]) >= 0)
            break;   // yes
   
         // no
         heap[i] = heap[ci]; // move child up
         i = ci;             // move down a level
         ci *= 2;
      }
      heap[i] = y;
   }

   /** test program */
   public static void main(String [] args)
   {
      MaxHeapWithChangeMax h = new MaxHeapWithChangeMax(4);
      h.changeMax(new Integer(1));
      h.put(new Integer(10));
      h.put(new Integer(20));
      h.put(new Integer(5));
      h.put(new Integer(15));
      h.put(new Integer(30));
      h.put(new Integer(15));
      h.put(new Integer(9));
      h.put(new Integer(11));

      System.out.println("Elements in array order are");
      System.out.println(h);
      System.out.println();

      // test change max
      System.out.println("The max element is " + h.getMax());
      h.changeMax(new Integer(2));
      System.out.println("The max element has been replaced with 2 ");
      System.out.println("The new max element is " + h.getMax());
      System.out.println("Elements in array order are");
      System.out.println(h);
      System.out.println();

      h.changeMax(new Integer(50));
      System.out.println("The max element has been replaced with 50 ");
      System.out.println("The new max element is " + h.getMax());
      System.out.println("Elements in array order are");
      System.out.println(h);
   }
}
