   
/** max heap with an enumerator */

package dataStructures;

import java.util.*;

public class MaxHeapWithIterator extends MaxHeap
{
   // constructors
   /** create a heap with the given initial capacity */
   public MaxHeapWithIterator(int initialCapacity)
      {super(initialCapacity);}
   
   /** create a heap with initial capacity 10 */
   public MaxHeapWithIterator()
      {this(10);}

   /** create and return an enumerator */
   public Iterator iterator()
      {return new HeapIterator();}

   private class HeapIterator implements Iterator
   {
      // data members
      private int nextIndex;  // index of next element
   
      // constructor
      public HeapIterator()
         {nextIndex = 1;}
   
      // methods
      /** @return true iff list has more iterator */
      public boolean hasNext()
         {return nextIndex <= size;}
   
      /** @return next element in array
        * @throws NoSuchElementException
        * when there is no next element */
      public Object next()
      {
         if (nextIndex <= size)
            return heap[nextIndex++];
         else throw new NoSuchElementException("No next element");
      }

      /** unsupported method */
      public void remove()
      {
         throw new UnsupportedOperationException
                   ("remove not supported");
      }   
   }

   /** test program */
   public static void main(String [] args)
   {
      MaxHeapWithIterator h = new MaxHeapWithIterator();
      Comparable [] z = new Comparable [10];
      for (int i = 1; i < 10; i++)
         z[i] = new Integer(i);
      h.initialize(z, 9);
      System.out.println("Elements in array order are");
      System.out.println(h);

      // test the iterator
      System.out.println("\nIterator for heap yields");
      Iterator ih = h.iterator();
      while (ih.hasNext())
         System.out.print(ih.next() + " ");
      System.out.println();
   }
}
