   
/** weight biased leftist trees */

package dataStructures;

public class MaxWblt implements MaxPriorityQueue
{
   // top-level nested class 
   private static class WbltNode
   {
      // data members
      Comparable element;
      int weight;            // subtree weight
      WbltNode leftChild;    // pointer to left subtree
      WbltNode rightChild;   // pointer to right subtree

      // constructor
      private WbltNode(Comparable theElement, int theWeight)
      {
         element = theElement;
         weight = theWeight;
      }
   }

   // data members of MaxWblt
   WbltNode root;   // pointer to tree root
   int size;        // number of elements in tree

   // only default constructor available

   // methods
   /** @return true iff the tree is empty */
   public boolean isEmpty()
      {return size == 0;}

   /** @return number of elements in the tree */
   public int size()
      {return size;}

   /** @return maximum element
     * @return null if the tree is empty */
   public Comparable getMax()
   {
      if (size == 0)
         return null;
      else
         return root.element;
   }

   /** meld the max leftist trees this and x
     * on exit, this is the result */
   public void meld(MaxWblt x)
      {root = meld(root, x.root);}
   
   /** meld the leftist trees with roots x and y
     * @return pointer to root of resulting tree */
   private static WbltNode meld(WbltNode x, WbltNode y)
   {
      if (y == null)
         return x;   // y is empty
      if (x == null)
         return y;   // x is empty
   
      // neither is empty, swap x and y if necessary
      if (x.element.compareTo(y.element) < 0)
      {// swap x and y
         WbltNode t = x;
         x = y;
         y = t;
      }
      // now x.element >= y.element
      // update weight of x
      x.weight += y.weight;
   
      WbltNode u = x;
      // need to meld u.rightChild and y
      while (true)
      {// neither u nor y is empty, meld u.rightChild and y
         // see if subtrees of u are to be eventually
         // swapped as a result of this meld
         if (u.leftChild == null)
         {// left subtree empty
          // right subtree must also be empty
            u.leftChild = y;
            return x;
          }
   
         if (u.rightChild == null)
         {// right subtree empty
            // see if y should be left or right subtree of u
            if (u.leftChild.weight >= y.weight)
               u.rightChild = y;
            else
            {
               u.rightChild = u.leftChild;
               u.leftChild = y;
            }
            return x;
         }
   
         // set z to root of meld(u.rightChild,y)
         WbltNode z = u.rightChild;
         if (z.element.compareTo(y.element) < 0)
         {
            z = y;
            y = u.rightChild;
         }
         // z will be root of subtree, update weight of z
         z.weight += y.weight;

         // set subtrees of u
         if (u.leftChild.weight >= z.weight) 
            // z will be right child
            u.rightChild = z;
         else
         {
            u.rightChild = u.leftChild;
            u.leftChild = z;
         }

         u = z;
         // proceed to meld u.rightChild and y
      }
   }

   /** put theElement into the heap */
   public void put(Comparable theElement)
   {
      WbltNode q = new WbltNode(theElement, 1);
      // meld q and original tree
      root = meld(root, q);
      size++;
   }
   
   /** remove max element and return it */
   public Comparable removeMax()
   {
      if (size == 0)
         return null;   // tree is empty
   
      // tree not empty
      Comparable x = root.element;  // save max element
      root = meld(root.leftChild, root.rightChild);
      size--;
      return x;
   }
   
   /** initialize leftist tree to elements in array thetheElements */
   public void initialize(Comparable [] theElements, int theSize)
   {
      size = theSize;
      ArrayQueue q = new ArrayQueue(size);
      // initialize queue of trees
      for (int i = 1; i <= size; i++)
         // create trees with one node each
         q.put(new WbltNode(theElements[i], 1));
   
      // repeatedly meld from queue q
      for (int i = 1; i <= size - 1; i++)
      {  // remove and meld two trees from the queue
         WbltNode b = (WbltNode) q.remove();
         WbltNode c = (WbltNode) q.remove();
         b = meld(b, c);
         // put melded tree on queue
         q.put(b);
      }
   
      if (size > 0)
         root = (WbltNode) q.remove();
   }
   
   /** postorder listing */
   private static StringBuffer s;
   public String toString()
   {
      s = new StringBuffer(); 
      s.append("The " + size + " elements, in postorder, are [");
      if (root == null)
         s.append("]");
      else
      {
         postOrderString(root);
         s.setCharAt(s.length() - 2, ']');  // replace last comma
      }
      return new String(s);
   }

   /** actual postorder string generator */
   private static void postOrderString(WbltNode t)
   {
      if (t != null)
      {
         postOrderString(t.leftChild);
         postOrderString(t.rightChild);
         s.append(t.element + ", ");
      }
  }

   /** test program */
   public static void main(String [] args)
   {
      // test constructor and put
      MaxWblt h = new MaxWblt();
      h.put(new Integer(10));
      h.put(new Integer(20));
      h.put(new Integer(5));

      // test toString
      System.out.println("Elements in postorder are");
      System.out.println(h);
      System.out.println();

      h.put(new Integer(15));
      h.put(new Integer(30));

      System.out.println(h);
      System.out.println();

      // test remove max
      System.out.println("The max element is " + h.getMax());
      System.out.println("Deleted max element " + h.removeMax());
      System.out.println("Deleted max element " + h.removeMax());
      System.out.println(h);
      System.out.println();

      // test initialize
      Comparable [] z = new Comparable [10];
      for (int i = 1; i < 10; i++)
         z[i] = new Integer(i);
      h.initialize(z, 9);
      System.out.println(h);
   }
}
