   
/** min heap extended to include the method changeMin */

package dataStructures;

public class MinHeapWithChangeMin extends MinHeap
{
   // constructors
   /** create a heap with the given initial capacity */
   public MinHeapWithChangeMin(int initialCapacity)
      {super(initialCapacity);}
   
   /** create a heap with initial capacity 10 */
   public MinHeapWithChangeMin()
      {super(10);}

   /** change the min element to theElement */
   public void changeMin(Comparable theElement)
   {
      if (size == 0)
      {// heap is empty, make a one element heap
         if (heap.length < 2)
            // allocate larger array
            heap = new Comparable [2];
         heap[1] = theElement;
         size = 1;
      }
         
      // nonempty heap, replace heap[1]
      Comparable y = theElement;      // last element
   
      // find place for y starting at root
      int i = 1,  // current node of heap
          ci = 2; // child of i
      while (ci <= size)
      {
         // heap[ci] should be smaller child of i
         if (ci < size &&
             heap[ci].compareTo(heap[ci + 1]) > 0)
            ci++;
   
         // can we put y in heap[i]?
         if (y.compareTo(heap[ci]) <= 0)
            break;   // yes
   
         // no
         heap[i] = heap[ci]; // move child up
         i = ci;             // move down a level
         ci *= 2;
      }
      heap[i] = y;
   }

   /** test program */
   public static void main(String [] args)
   {
      MinHeapWithChangeMin h = new MinHeapWithChangeMin(4);
      h.changeMin(new Integer(1));
      h.put(new Integer(10));
      h.put(new Integer(20));
      h.put(new Integer(5));
      h.put(new Integer(15));
      h.put(new Integer(30));
      h.put(new Integer(15));
      h.put(new Integer(9));
      h.put(new Integer(11));

      System.out.println("Elements in array order are");
      System.out.println(h);
      System.out.println();

      // test change min
      System.out.println("The min element is " + h.getMin());
      h.changeMin(new Integer(25));
      System.out.println("The min element has been replaced with 25");
      System.out.println("The new min element is " + h.getMin());
      System.out.println("Elements in array order are");
      System.out.println(h);
      System.out.println();

      h.changeMin(new Integer(8));
      System.out.println("The min element has been replaced with 8");
      System.out.println("The new min element is " + h.getMin());
      System.out.println("Elements in array order are");
      System.out.println(h);
   }
}
