
/** insertion, selection, bubble, and rank sort methods
  * to sort a simulated chain */

package dataStructures;

import wrappers.*;
import utilities.*;

public class SimulatedChainWithSortMethods extends SimulatedChain
{
   // constructors
   public SimulatedChainWithSortMethods(int initialCapacity)
      {super(initialCapacity);}

   public SimulatedChainWithSortMethods()
      {super();}

   /** insertion sort */
   public void insertionSort()
   {
      int sFirstNode = -1;  // start of sorted chain

      // go down the unsorted chain inserting nodes into the sorted
      // chain that begins at sFirst
      while (firstNode != -1)
      {// a node remains in the unsorted chain
         // save pointer to next node in unsorted chain
         int nextNode = S.node[firstNode].next;
   
         // find place to insert firstNode in sorted chain
         int p = sFirstNode,  // current node
             tp = -1;         // tp trails p
         while (p != -1 &&
                ((Comparable) S.node[p].element).
                  lessThan(S.node[firstNode].element))
         {// move tp and p one node right
            tp = p;
            p = S.node[p].next;
         }
   
         // insert firstNode after tp
         if (tp != -1)
         {// not at front
            S.node[tp].next = firstNode;
            S.node[firstNode].next = p;
         }
         else
         {// insert at front
             sFirstNode = firstNode;
             S.node[firstNode].next = p;
         }
         
         // update firstNode
         firstNode = nextNode;
      }

      firstNode = sFirstNode;
   
      return;
   }

   /** selection sort */
   public void selectionSort()
   {
      for (int i = size; i > 1; i--)
      {
         // find max object from first i nodes
         Comparable currentMax = (Comparable) S.node[firstNode].element;
         int maxNode = firstNode;
         int currentNode = firstNode;
         for (int j = 1; j < i; j++)
         {
            int nextNode = S.node[currentNode].next;
            if (currentMax.lessThan(S.node[nextNode].element))
            {// found a larger element
               currentMax = (Comparable) S.node[nextNode].element;
               maxNode = nextNode;
            }

            // move to next node
            currentNode = nextNode;
         }

         // move max object to right end
         S.node[maxNode].element = S.node[currentNode].element;
         S.node[currentNode].element = currentMax;
      }
   }

   // bubble sort
   /** bubble largest element in first n nodes to right */
   private void bubble(int n)
   {
      int currentNode = firstNode;
      for (int i = 0; i < n - 1; i++)
      {
         int nextNode = S.node[currentNode].next;
         if (((Comparable) S.node[currentNode].element)
               .greaterThan(S.node[nextNode].element))
         {// swap elements
            Object temp = S.node[currentNode].element;
            S.node[currentNode].element = S.node[nextNode].element;
            S.node[nextNode].element = temp;
         }
         currentNode = nextNode;
      }
   }

   /** sort the array a using the bubble sort method */
   public void bubbleSort()
   {
      for (int i = size; i > 1; i--)
         bubble(i);
   }

   /** sort the chain using the rank sort method */
   public void rankSort()
   {
      if (size == 0)
         return;  // empty chain
   
      // r[i] will be rank of element i
      int [] r = new int [size];
   
      // compute ranks by comparing all pairs of elements
      int currentNode = firstNode;
      for (int i = 0; i < size; i++)
      {// compare S.node[currentNode].element with all elements
         int compareNode = firstNode;
         for (int j = 0; j < i; j++)
         {// S.node[currentNode].element is element i and
          // and S.node[compareNode].element is element j, j < i
            if (((Comparable) S.node[compareNode].element).lessThanOrEqual
                                     (S.node[currentNode].element))
               r[i]++;
            else
               r[j]++;
            compareNode = S.node[compareNode].next;
         }
         currentNode = S.node[currentNode].next;
      }
   
      // distribute nodes to bins by rank
      int [] bin = new int [size];
      currentNode = firstNode;
      for (int i = 0; i < size; i++)
      {
         bin[r[i]] = currentNode;
         currentNode = S.node[currentNode].next;
      }
   
      // collect from bins
      firstNode = bin[0];
      int lastNode = firstNode;  // last node on chain
      for (int i = 1; i < size; i++)
         lastNode = S.node[lastNode].next = bin[i];
      S.node[lastNode].next = -1;
   }

   /** test program */
   public static void main(String [] args)
   {
      SimulatedChain.S = new SimulatedSpace2(100);
      SimulatedChainWithSortMethods x = new SimulatedChainWithSortMethods();
      x.insertElementAt(new MyInteger(3), 0);
      x.insertElementAt(new MyInteger(2), 1);
      x.insertElementAt(new MyInteger(4), 2);
      x.insertElementAt(new MyInteger(1), 3);
      x.insertElementAt(new MyInteger(6), 4);
      x.insertElementAt(new MyInteger(9), 5);
      x.insertElementAt(new MyInteger(8), 6);
      x.insertElementAt(new MyInteger(7), 7);
      x.insertElementAt(new MyInteger(5), 8);
      x.insertElementAt(new MyInteger(0), 9);
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.insertionSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);

      x = new SimulatedChainWithSortMethods();
      x.insertElementAt(new MyInteger(3), 0);
      x.insertElementAt(new MyInteger(2), 1);
      x.insertElementAt(new MyInteger(4), 2);
      x.insertElementAt(new MyInteger(1), 3);
      x.insertElementAt(new MyInteger(6), 4);
      x.insertElementAt(new MyInteger(9), 5);
      x.insertElementAt(new MyInteger(8), 6);
      x.insertElementAt(new MyInteger(7), 7);
      x.insertElementAt(new MyInteger(5), 8);
      x.insertElementAt(new MyInteger(0), 9);
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.selectionSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);

      x = new SimulatedChainWithSortMethods();
      x.insertElementAt(new MyInteger(3), 0);
      x.insertElementAt(new MyInteger(2), 1);
      x.insertElementAt(new MyInteger(4), 2);
      x.insertElementAt(new MyInteger(1), 3);
      x.insertElementAt(new MyInteger(6), 4);
      x.insertElementAt(new MyInteger(9), 5);
      x.insertElementAt(new MyInteger(8), 6);
      x.insertElementAt(new MyInteger(7), 7);
      x.insertElementAt(new MyInteger(5), 8);
      x.insertElementAt(new MyInteger(0), 9);
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.bubbleSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);

      x = new SimulatedChainWithSortMethods();
      x.insertElementAt(new MyInteger(3), 0);
      x.insertElementAt(new MyInteger(2), 1);
      x.insertElementAt(new MyInteger(4), 2);
      x.insertElementAt(new MyInteger(1), 3);
      x.insertElementAt(new MyInteger(6), 4);
      x.insertElementAt(new MyInteger(9), 5);
      x.insertElementAt(new MyInteger(8), 6);
      x.insertElementAt(new MyInteger(7), 7);
      x.insertElementAt(new MyInteger(5), 8);
      x.insertElementAt(new MyInteger(0), 9);
   
      // output elements to be sorted
      System.out.println("The elements are " + x);

      // sort the elements
      x.rankSort();

      // output in sorted order
      System.out.println("The sorted order is " + x);
   }
}
