
package dataStructures;

public class SkipListWithDuplicates extends SkipList
{
   // constructor
   public SkipListWithDuplicates(Comparable largeKey,
                                 int maxElements, float theProb)
      {super(largeKey, maxElements, theProb);}

   /** insert an element with the specified key
     * do not overwrite old element if there is already an
     * element with the given key 
     * overrides SkipList.put
     * @returns null */
   public Object put(Object theKey, Object theElement)
   {
      if (tailKey.compareTo(theKey) < 0) // key too large
         throw new IllegalArgumentException
                   ("key is too large");
      
      // new element will be placed just after node p
      SkipNode p = search(theKey);
   
      // determine level for new node
      int lev = level(); // level of new node
      // fix lev to be <= levels + 1
      if (lev > levels)
      {
         lev = ++levels;
         last[lev] = headNode;
      }
   
      // get and insert new node just after p
      SkipNode y = new SkipNode (theKey, theElement, lev + 1);
      for (int i = 0; i <= lev; i++)
      {// insert into level i chain
         y.next[i] = last[i].next[i];
         last[i].next[i] = y;
      }
      size++;
      return null;
   }
   
   /** test program */
   public static void main(String [] args)
   {
      // test constructor
      SkipListWithDuplicates x =
          new SkipListWithDuplicates(new Integer(1001), 100, (float) 0.5);

      // test put
      int n = 20;
      for (int i = 1; i <= n; i++)
         x.put(new Integer(2 * i), new Integer(i));
      System.out.println("The list is");
      System.out.println(x);

      for (int i = 1; i <= n + 1; i++)
         x.put(new Integer(2 * i), new Integer(n + i));
      System.out.println("The list is\n" + x);

      // test get
      System.out.println("element " + x.get(new Integer(1))
                         + " has key 1");
      System.out.println("element " + x.get(new Integer(2))
                         + " has key 2");
      System.out.println("element " + x.get(new Integer(6))
                         + " has key 6");

      // test remove
      for (int i = 1; i <= n + 1; i++)
         System.out.println("removed " + x.remove(new Integer(2 * i)));
      System.out.println("The list is\n" + x);
   }
}
