
/** two lower-triangle matrixes stored in a single two-dimensional array */

package dataStructures;

import utilities.*;
import wrappers.*;

public class TwoInOne
{
   // data members
   int rows;              // matrix dimension
   Object zero;           // zero element
   Object [][] element;   // element array

   // constructor
   public TwoInOne(int theRows, Object theZero)
   {
      // validate theRows
      if (theRows < 1)
         throw new IllegalArgumentException
                   ("number of rows must be > 0");
   
      rows = theRows;
      zero = theZero;

      // create and initialize the array element
      element = new Object [rows + 1][rows];
      for (int i = 0; i <= rows; i++)
         for (int j = 0; j < rows; j++)
         element[i][j] = zero;

   }
   
   // methods
   
   /** @throws IndexOutOfBoundsException when i < 1
     * or j < 1 or i > rows or j > rows */
   void checkIndex(int i, int j)
   {
      if (i < 1 || j < 1 || i > rows || j > rows)
         throw new IndexOutOfBoundsException
                   ("i = " + i + " j = " + j +
                    " rows = " + rows + " cols = " + rows);
   }

   /** @return the (i,j) element of matrix theMatrix
     * @throws IndexOutOfBoundsException when i or j invalid
     * @throws IllegalArgumentException when theMatrix not 1 or 2 */
   public Object get(int theMatrix, int i, int j)
   {
      // validate theMatrix
      if (theMatrix < 1 || theMatrix > 2)
         throw new IllegalArgumentException
               ("matrix identifier must be 1 or 2");

      // validate index
      checkIndex(i, j);

      // (i,j) in lower triangle iff i >= j
      if (i >= j)
         // get for appropriate matrix
         if (theMatrix == 1)
            return element[i][j - 1];
         else
            // second matrix
            return element[j - 1][i - 1];
      else
         // (i,j) is in upper triangle
         return zero;
   }
   
   /** set the (i,j) element of matrix theMatrix to newValue
     * @throws IndexOutOfBoundsException when i or j invalid
     * @throws IllegalArgumentException when theMatrix not 1 or 2 */
   public void set(int theMatrix, int i, int j, Object newValue)
   {
      // validate theMatrix
      if (theMatrix < 1 || theMatrix > 2)
         throw new IllegalArgumentException
               ("matrix identifier must be 1 or 2");

      // validate i and j
      checkIndex(i, j);

      // store newValue
      // (i,j) in lower triangle iff i >= j
      if (i >= j)
         // save based on matrix
         if (theMatrix == 1)
            // first matrix
            element[i][j - 1] = newValue;
         else
            // second matrix
            element[j - 1][i - 1] = newValue;
      else
         // not in the lower triangle
         if (!((Zero) newValue).equalsZero())
                   throw new IllegalArgumentException
                       ("elements not in lower triangle must be zero");
   }
   
   
   /** test program */
   public static void main(String [] args)
   {
      // create the matrix
      TwoInOne x = new TwoInOne(10, new MyInteger(0));

      // set a few elements
      x.set(1, 1, 1, new MyInteger(22));
      x.set(2, 3, 2, new MyInteger(12));
      x.set(2, 5, 5, new MyInteger(44));
      x.set(1, 2, 5, new MyInteger(0));
      x.set(1, 7, 4, new MyInteger(55));

      // retrieve a few elements
      System.out.println(x.get(1, 7, 8));
      System.out.println(x.get(2, 5, 5));
      System.out.println(x.get(1, 1, 1));
      System.out.println(x.get(1, 1, 2));
      System.out.println(x.get(2, 10, 1));
      System.out.println(x.get(1, 7, 4));
   }
}
